/* MAchart.c - chart functions for MoA
 *
 * create : 95.6.24 (Sat), bgjang@csone - rewritten.
 * modify :
 * 95.6.26 (Mon), bgjang@csone - version 0.5 alpha
 * 95.7.18-31 (Mon), bgjang
 *    MAchartDeleteByID(), MAchartGetIth(), MAedgeLmRmOf()
 *
 *
 *-------------------------------------------------------
 *
 * All functions of which name starts with "MA" will handles
 * error occured in itself. So, if there is some return value
 * then always it represents that some works are not done or
 * there is no proper one to be returned.
 *
 */
#include <stdio.h>
#include <token.h>
#include "common.h"
#include "pos.h"
#include "chart.h"
#include "MAchart.h"
#include "MAerror.h"

#if defined(LOG_STATISTIC)
extern int maxUsedChartNum;
int curUsedChartNum = 0;
#endif

void MAchartInit()
{
    chartInit();
}

void MAchartCreate( pId )
    int *pId;
{
    int r;
    int chartID;

    setFuncName( "MAchartCreate" );
    r = chartCreate( &chartID );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );

    *pId = chartID;

#if defined(LOG_STATISTIC)
    curUsedChartNum++;
    maxUsedChartNum = MAX(maxUsedChartNum, curUsedChartNum);
#endif
}

void MAchartDestroy( id )
    int id;
{
    int r;

    setFuncName( "MAchartDestroy" );
    r = chartDestroy( id );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );

#if defined(LOG_STATISTIC)
    curUsedChartNum--;
#endif
}

void MAchartCopy( fromID, toID )
    int fromID;
    int toID;
{
    int r;

    setFuncName( "MAchartCopy" );
    r = chartCopy( fromID, toID );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

#if defined(DEBUG)

void MAchartDump( id, isHH, isHT, isHE )
    int id;
    int isHH, isHT, isHE;
{
    chartDump( id, isHH, isHT, isHE );
}

#endif

void MAchartInsert( id, headV, tailV, str, epos, freq, pEdgeID )
    int id;
    int headV, tailV;
    char *str;
    enum EPOS epos;
    float freq;
    int *pEdgeID;
{
    int r;

    setFuncName( "MAchartInsert" );
    r = chartInsertEdge( id, headV, tailV, str, epos, freq, pEdgeID );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

/* insert functions for S-Type chart.
 */
void MAchartInsertS( id, headV, type, kimmo, pTailV )
    int id;
    int headV;
    enum tokenType type;
    char *kimmo;
    int *pTailV;
{
    register int i;
    int r;
    int tailV;
    int e;

    setFuncName( "MAchartInsertS" );

    switch( type ) {

    /* T_KOR
     * all characters are seperated by one character and
     * make one edge for one character.
     */
    case tokenK :
        for( i=0; i<strlen(kimmo); i++ ) {
            char oneChar[2];
            oneChar[0] = *(kimmo+i);
            oneChar[1] = '\0';
            MAchartInsert( id, headV, headV+VII, oneChar, NONE, -1.0, &e );

            headV += VII;
        }
        *pTailV = headV;
        return;
        break;

    /* T_ENG, T_NUM, T_SYM
     * all characters are stored in whole.
     */
    case tokenS :
    case tokenE :
    case tokenN :
        MAchartInsert( id, headV, headV+VII, kimmo, NONE, -1.0, &e );
        *pTailV = headV + VII;
        return;
    }
}

/* insert functions for T-Type functions
 *
 * IorE value will be stored in edge's freq variable
 * so we must use MAedgeIorEOf() functions to get
 * this value.
 */
void MAchartInsertT( id, headV, tailV, IorE, lm, rm )
    int id;
    int headV, tailV;
    enum IE IorE;
    char lm, rm;
{
    int e;
    char str[8];

    /* left marker and right marker is stored in str data */
    sprintf( str, "%c %c", lm, rm );

    MAchartInsert( id, headV, tailV, str, (enum EPOS) IorE, 0.0, &e );
}

void MAchartPutIrrApplied( id, irrApplied )
    int id;
    long irrApplied;
{
    int r;
    r = chartPutIrrApplied( id, irrApplied );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

void MAchartGetIrrApplied( id, pIrrApplied )
    int id;
    long *pIrrApplied;
{
    int r;
    r = chartGetIrrApplied( id, pIrrApplied );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

/* delete the first edges of which head vertex ID is
 * headV. Notice that if there are two edges of which
 * head vertex ID is headV, then nobody can know which one
 * will be deleted.
 */
int MAchartDeleteByHeadV( id, headV )
    int id;
    int headV;
{
    int r;
    int edgeID;

    setFuncName( "MAchartDeleteByHeadV" );

    r = MAchartGetByHeadV( id, headV, &edgeID );
    if( r == FALSE ) return FALSE;

    r = chartDelete( id, edgeID );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
    return TRUE;
}

int MAchartDeleteByID( id, edgeID )
    int id;
    int edgeID;
{
    int r;

    r = chartDelete( id, edgeID );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
    return TRUE;
}

/* MAchartGet*() functions --------------------------------------*/

/* all get functions return the edge ID of wanted edge
 * and the contents related to edge can be obtained by
 * edge*() functions by edge ID
 *
 * moreover, to connect two edges, we must use two edge ID
 */
int MAchartGetIth( id, ith, pEdgeID )
    int id;
    int ith;
    int *pEdgeID;
{
    int r;
    r = chartGetIth( id, ith, pEdgeID );
    if( r == C_NO_DATA ) return FALSE;
    return TRUE;
}

int MAchartGetByHeadV( id, headV, pEdgeID )
    int id;
    int headV;
    int *pEdgeID;   // OUT
{
    return MAchartGetIthByHeadV( id, 0, headV, pEdgeID );
}

int MAchartGetIthByHeadV( id, ith, headV, pEdgeID )
    int id;
    int ith;
    int headV;
    int *pEdgeID;   // OUT
{
    int r;

    setFuncName( "MAchartGetIthByHeadV" );
    r = chartGetIthByHeadV( id, ith, headV, pEdgeID );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
    if( r == C_NO_DATA ) return FALSE;
    return TRUE;
}

/* like MAchartGetByHeadV() function, the first one will
 * be retured.
 */
int MAchartGetByTailV( id, tailV, pEdgeID )
    int id;
    int tailV;
    int *pEdgeID;   // OUT
{
    return MAchartGetIthByTailV( id, 0, tailV, pEdgeID );
}

int MAchartGetIthByTailV( id, ith, tailV, pEdgeID )
    int id;
    int ith;
    int tailV;
    int *pEdgeID;   // OUT
{
    int r;

    setFuncName( "MAchartGetIthByHeadV" );
    r = chartGetIthByTailV( id, ith, tailV, pEdgeID );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
    if( r == C_NO_DATA ) return FALSE;
    return TRUE;
}

/* this only get the first edges from chart of which
 * head vertex ID matches to the previous tail vertex ID.
 * and get the concatenations result.
 *
 * CAUTION : if there are more than two edges of which head
 * vertex ID is same, then the behavior can not be guessed.
 */
int MAchartGetStringByInterval( id, headV, tailV, pString )
    int id;
    int headV;
    int tailV;
    char *pString;   // OUT
{
    int r;
    int edgeID;
    char tStr[128];
    int hv, tv;

    strcpy( pString, "" );
    hv = headV;
    while(1) {
        r = MAchartGetByHeadV( id, hv, &edgeID );
        if( r == FALSE ) return FALSE;

        MAedgeTailVOf( id, edgeID, &tv );
        MAedgeStrOf( id, edgeID, tStr );
        strcat( pString, tStr );
        if( tv == tailV ) return TRUE;
        hv = tv;
    }
}

/* Notice that MAchartConnectEdges( id, A, B ) and
 * MAchartConnectEdges( id, B, A ) is different.
 */
void MAchartConnectEdges( id, edgeIDA, edgeIDB )
    int id;
    int edgeIDA, edgeIDB;
{
    int r;

    setFuncName( "MAchartConnectEdges" );
    r = chartConnect( id, edgeIDA, edgeIDB );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}


/* -1 : edgeIDA is included in edgeIDB
 *  1 : edgeIDB       "        edgeIDA
 *  0 : not included
 */
int MAchartIsIncluded( id, edgeIDA, edgeIDB )
    int id;
    int edgeIDA, edgeIDB;
{
    int hvA, tvA, hvB, tvB;
    MAedgeHeadVOf( id, edgeIDA, &hvA );
    MAedgeTailVOf( id, edgeIDA, &tvA );
    MAedgeHeadVOf( id, edgeIDA, &hvB );
    MAedgeTailVOf( id, edgeIDA, &tvB );

    if( hvA <= hvB && tvB <= tvA ) return 1;
    else if( hvB <= hvA && tvA <= tvB ) return -1;
    else return 0;
}


/* edge*() functions --------------------------------------------*/

/* these function sets are used to access to the values related to
 * an edges in chart identified by edge ID.
 */

void MAedgeStrOf( id, edgeID, str )
    int id, edgeID;
    char *str;
{
    int r;
    setFuncName( "MAedgeStrOf" );
    r = edgeStrOf( id, edgeID, str );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

void MAedgeEPOSOf( id, edgeID, pEpos )
    int id, edgeID;
    enum EPOS *pEpos;
{
    int r;
    setFuncName( "MAedgeEPOSOf" );
    r = edgeEPOSOf( id, edgeID, pEpos );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

void MAedgePOSOf( id, edgeID, pPos )
    int id, edgeID;
    enum POS *pPos;
{
    int r;
    setFuncName( "MAedgePOSOf" );
    r = edgePOSOf( id, edgeID, pPos );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

void MAedgeFreqOf( id, edgeID, pFreq )
    int id, edgeID;
    float *pFreq;
{
    int r;
    setFuncName( "MAedgeFreqOf" );
    r = edgeFreqOf( id, edgeID, pFreq );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

void MAedgeIorEOf( id, edgeID, pIorE )
    int id, edgeID;
    enum IE *pIorE;
{
    int r;
    setFuncName( "MAedgeIorEOf" );
    r = edgeIorEOf( id, edgeID, pIorE );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

void MAedgeLmRmOf( id, edgeID, pLm, pRm )
    int id, edgeID;
    char *pLm, *pRm;
{
    int r;
    setFuncName( "MAedgeLmRmOf" );
    r = edgeLmRmOf( id, edgeID, pLm, pRm );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}


void MAedgeHeadVOf( id, edgeID, pHeadV )
    int id, edgeID;
    int *pHeadV;
{
    int r;
    setFuncName( "MAedgeHeadVOf" );
    r = edgeHeadVOf( id, edgeID, pHeadV );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

void MAedgeTailVOf( id, edgeID, pTailV )
    int id, edgeID;
    int *pTailV;
{
    int r;
    setFuncName( "MAedgeTailVOf" );
    r = edgeTailVOf( id, edgeID, pTailV );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
}

int MAedgeIthNextOf( id, edgeID, ith, pNext )
    int id, edgeID;
    int ith, *pNext;
{
    int r;
    setFuncName( "MAedgeIthNextOf" );
    r = edgeIthNextOf( id, edgeID, ith, pNext );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
    if( r == C_NO_DATA ) return FALSE;
    return TRUE;
}

int MAedgeIthPrevOf( id, edgeID, ith, pPrev )
    int id, edgeID;
    int ith, *pPrev;
{
    int r;
    setFuncName( "MAedgeIthPrevOf" );
    r = edgeIthPrevOf( id, edgeID, ith, pPrev );
    if( r < 0 )
        MAerror( END, E_CHART + r, __FILE__, __LINE__, "%s", chartES );
    if( r == C_NO_DATA ) return FALSE;
    return TRUE;
}
