/* header file for chart Data structure
 *
 * create :
 * modify :
 * 95.6.13 (Tue), bgjang@csone - to accomodate the changes of types
 * 95.6.25 (Sun), bgjang@csone - function declarations
 *
 *
 */

#ifndef __MACHART_H__
#define __MACHART_H__

/* Chart Type
 *
 * according to the method to use variable field,
 * there is four kind of edge Pool.
 *
 * 1. S-type : syllabic edge used POS
 * 2. R-type : result edge
 * 3. T-type : temporarily used edge for Irregullar/Exceptional edges
 * 4. P-type : alike syllabic edge but use EPOS for procedure to search
 *             normal dictionary and make R-type chart.
 *
 * 0. N-type : none
 *
 */
enum chartType { chartTypeN, chartTypeS, chartTypeR, chartTypeT, chartTypeP };


/* in S-type chart, each head vertex ID and tail vertex ID
 * have some difference and this value is uniform for all edges.
 * And the value is determined by the following VERTEX_ID_INTERVAL
 * constant.
 *
 * And this value effect the performance of Hash table in
 * chart structure, so must be determined with caution.
 */
#define VERTEX_ID_INTERVAL    7
#define VII   VERTEX_ID_INTERVAL


/* IorE type
 *
 * Irrgular/Exceptional distinguishment in T-type chart
 * Irrgular and Exceptional.
 *
 * IEpure : the strings are consisted of only symbols.
 * IEcomplex : the strings are consisted of symbols and korean
 */
enum IE { IEIrr, IEExce, IENone };



/* functions declarations --------------------------------------*/

extern void MAchartInit();
extern void MAchartCreate(/* pId */);
extern void MAchartDestroy(/* id */);
extern void MAchartCopy(/* fromID, toID */);
#if defined(DEBUG)
extern void MAchartDump(/* id */);
#endif
extern void MAchartInsert(/* id, headV, tailV, str, epos, freq, pEdgeID */);
extern void MAchartInsertS(/* id, headV, type, kimmo */);
extern void MAchartInsertT(/* id, headV, tailV, IorE */);
extern int  MAchartDeleteByHeadV(/* id, headV */);
extern int  MAchartDeleteByID(/* id, edgeID */);
extern int  MAchartGetIth(/* id, ith, pEdgeID */);
extern int  MAchartGetByHeadV(/* id, headV, pEdgeID */);
extern int  MAchartGetIthByHeadV(/* id, ith, headV, pEdgeID */);
extern int  MAchartGetByTailV(/* id, tailV, pEdgeID */);
extern int  MAchartGetIthByTailV(/* id, ith, tailV, pEdgeID */);
extern int  MAchartGetStringByInterval(/* id, headV, tailV, pString */);
extern void MAchartConnectEdges(/* id, edgeIDA, edgeIDB */);

extern void MAedgeStrOf(/* id, edgeID, str */);
extern void MAedgeEPOSOf(/* id, edgeID, pEpos */);
extern void MAedgePOSOf(/* id, edgeID, pPos */);
extern void MAedgeFreqOf(/* id, edgeID, pFreq */);
extern void MAedgeIorEOf(/* id, edgeID, pIorE */);
extern void MAedgeLmRmOf(/* id, edgeID, pLm, pRm */);
extern void MAedgeHeadVOf(/* id, edgeID, pHeadV */);
extern void MAedgeTailVOf(/* id, edgeID, pTailV */);
extern int  MAedgeIthNextOf(/* id, edgeID, ith, pNext */);
extern int  MAedgeIthPrevOf(/* id, edgeID, ith, pPrev */);

#endif
