/* Each Subjob for MoA.
 *
 * create : 95.7.18 (Tue), bgjang
 *   originally, MoA.c includes all of these functions, but seperate
 *   those sub-jobs to this file for simplicity of main file.
 * modify :
 * 95.7.18 (Tue), bgjang - Unknown word processing
 * 95.7.19 - 31 (Mon), bgjang - tuning for several misc steps.
 *
 *
 */

#include <stdio.h>
#include <assert.h>
#include <fcntl.h>
#include <MAdict.h>
#include "common.h"
#include "pos.h"
#include "irrtbl.h"
#include "token.h"
#include "MAchart.h"
#include "MAerror.h"
#include "orderedSet.h"


/* staic variables -----------------------------------------------*/
char *irrStr[32] = {
   "Exce",  "N",     "N",     "N",    "N",     "N",      "N",        "N",
   "IRR_i", "IRR_w8","IRR_e", "IRR_a","IRR_we","IRR_wa",  "IRR_ye_", "N",
   "IRR_h8","IRR_9", "IRR_8", "N",    "IRR_H", "IRR_nela","IRR_gela","IRR_ye",
   "IRR_u", "IRR_le","IRR_l_","IRR_B","IRR_D", "IRR_S",   "IRR_L",   "IRR__"
};

static int irrUsed;       /* in irregular, irregular rule ? */
static int omittedUsed;   /* in irregular, omitted is used or not */
                          /* both are used in dfsMakePTypeChart() */


/* static functions declarations ---------------------------------*/
static void dfsMakePTypeChart(/* SchartID, TchartID, stackID, PchartID, headV, lastTailV, hStackID */);
static int searchIdict(/* PchartID, irrKind, midV, pPreV, pAPchartID */);
static void exceMakeEdges(/* PchartID, headV, tailV, listOfWords, hStackID */);
static void popAndDelete(/* hStackID, PchartID, bottomV */);
static int doNS(/* PcID, RcID, headV, lastTailV */);
static void dfsDoPrintResult(/* stackID, RchartID, edgeID, lastTailV */);
static void printIrrApplied(/* iA */);
static int dfsDoUWP(/* PchartID, RchartID, edgeID, stackID */);
static int checkSymbolByTailV(/* chartID, tailV */);
static int checkSymbolByHeadV(/* chartID, headV */);
static int findStrEdict(/* strings, pLm, pRm */);




/*------------------------------------------------------------------
 *
 * routines for each subjob
 * 
 * each sub-job routine handle all abnormal error and
 * die gracefully. Otherwise return.
 *
 *------------------------------------------------------------------
 */

/* return val :
 * 1 : if token is maden.
 * 0 : otherwise
 */
int tokenization()
{
    int r;

    /* tokenization for one sentence
     * if there is no data to be tokenized, then return 0
     * else return 1
     */
    r = tokenMake();
    if( r < 0 ) {
        printf("%s", tokenErrorStr);
        MAKDestroy();
        exit(1);
    }
    if( r != 0 ) {
#if defined(DEBUG)
        tokenDump();
#endif
        return 1;
    }
    if( r == 0 ) return 0;
}

/* return val :
 * 0 : if there is no word to be maden to a chart
 * 1 : otherwise
 */
int selectOneWordAndMakeChart( SchartID, pLastTailV )
    int SchartID;
    int* pLastTailV;
{
    register int i;
    int headV, tailV;
    int r;
    int wordToken;           // token number in currently selected word
    static int wordEnd = 0;

    // if cannot make more word, then init static var. and return zero.
    if( wordEnd >= tokenNumber ) {
        wordEnd = 0;
        return 0;
    }

    // select one word and make S-Type chart
    wordToken = 0;
    headV = 0;
    for( i=wordEnd; i<tokenNumber; i++ ) {
        char tLine[256];

        if( !(tokenList[i].tt == tokenS &&
              tokenList[i].string[0] == ' ' && wordToken != 0) )
        {
            switch( tokenList[i].tt ) {
            case tokenS :
            case tokenE :
            case tokenN :
                sprintf( tLine, "%c%s%c",
                  CTRL(N), tokenList[i].string, CTRL(O) );
                break;
            case tokenK :
                strcpy( tLine, tokenList[i].string );
                break;
            }

            assert( strlen(tLine) < 256 );

            // error handling in function.
            MAchartInsertS( SchartID, headV, tokenList[i].tt, tLine, &tailV );
            headV = tailV;
            *pLastTailV = headV;
            wordToken++;
        }
        else
            break;
    }
    if( wordToken == 0 ) return 0;
    wordEnd = i + 1;  // next token after ' ' token
#if defined(DEBUG)
    MAchartDump( SchartID, 0, 0, 0 );
#endif
    return 1;
}

void doIrrExecWordSearch( SchartID, TchartID, lastTailV )
    int SchartID;
    int TchartID;
    int lastTailV;
{
    register int i, j;
    int r;
    int curV, headV;
    int edgeID;
    char tStr[64], strings[128];
    char rm, lm;
    enum IE IorE;

    setFuncName( "doIrrExecWordSearch" );

    /* last to first */
    for( curV = lastTailV - VII; !(curV < 0); curV -= VII ) {

        strcpy( strings, "" );
        if( curV == 0 ) lm = '$';
        else {
            if( checkSymbolByTailV( SchartID, curV ) == TRUE ) lm = '*';
            else lm = '+';
        }

        /* first to last */
        for( headV = curV; headV < lastTailV; headV += VII ) {

            r = MAchartGetByHeadV( SchartID, headV, &edgeID );
            if( r == FALSE )
                MAerror( END, E_STYPECHART, __FILE__, __LINE__,
                    "S-type chart have no edge for headV %d.\n", headV );
            MAedgeStrOf( SchartID, edgeID, tStr );

            strcat( strings, tStr );

            if( headV + VII == lastTailV ) rm = '$';
            else {
                if( checkSymbolByHeadV( SchartID, headV + VII ) == TRUE ) rm = '*';
                else rm = '+';
            }

            /* check Irregular table */
            r = irrIsPatternIrr( strings );
            if( r == TRUE )
                MAchartInsertT( TchartID, curV, headV + VII, IEIrr, lm, rm );

            /* check Exceptional dictionary */
            r = findStrEdict( strings, &lm, &rm );
            if( r == TRUE )
                MAchartInsertT( TchartID, curV, headV + VII, IEExce, lm, rm );
        }
    }

    return;
}

/* check the edge of which tail vertex ID is tailV is
 * symbol or not. The result when there is two edges is
 * not known.
 */ 
static int checkSymbolByTailV( chartID, tailV )
    int chartID;
    int tailV;
{
    register int i;
    int e;
    char edgeStr[128];

    MAchartGetByTailV( chartID, tailV, &e );
    MAedgeStrOf( chartID, e, edgeStr );

    if( edgeStr[0] == CTRL(N) ) {
        for( i=0; i<strlen(edgeStr)-1; i++ )
            if( !isalpha(edgeStr[i]) ) return TRUE;
        return FALSE;
    }
    else return FALSE;
}

static int checkSymbolByHeadV( chartID, headV )
    int chartID;
    int headV;
{
    register int i;
    int e;
    char edgeStr[128];

    MAchartGetByHeadV( chartID, headV, &e );
    MAedgeStrOf( chartID, e, edgeStr );

    if( edgeStr[0] == CTRL(N) ) {
        for( i=0; i<strlen(edgeStr)-1; i++ )
            if( !isalpha(edgeStr[i]) ) return TRUE;
        return FALSE;
    }
    else return FALSE;
}


/* find some string is in E-Type dictionary.
 * if left marker or right marker is '$' and there is no proper
 * word, then change one marker to '*' and try again.
 *
 * the result will be in pLm, and pRm
 */
static int findStrEdict( strings, pLm, pRm )
    char* strings;
    char* pLm;
    char* pRm;
{ 
    int r;
    char lm, rm;
    char str[128];

    lm = *pLm; rm = *pRm;
    r = MAdictEGetData( EdictID, strings, lm, rm, str );
    if( r == TRUE ) goto R_TRUE;

    if( lm == '$' || rm == '$' ) {
        lm = '$'; rm = '*';
        r = MAdictEGetData( EdictID, strings, lm, rm, str );
        if( r == TRUE ) goto R_TRUE;

        lm = '*'; rm = '$';
        r = MAdictEGetData( EdictID, strings, lm, rm, str );
        if( r == TRUE ) goto R_TRUE;

        lm = '*'; rm = '*';
        r = MAdictEGetData( EdictID, strings, lm, rm, str );
        if( r == TRUE ) goto R_TRUE;

        goto R_FALSE;
    }
    else if( lm == '$' ) {
        lm = '*';
        r = MAdictEGetData( EdictID, strings, lm, rm, str );
        if( r == TRUE ) goto R_TRUE;

        goto R_FALSE;
    }
    else if( rm == '$' ) {
        rm = '*';
        r = MAdictEGetData( EdictID, strings, lm, rm, str );
        if( r == TRUE ) goto R_TRUE;

        goto R_FALSE;
    }
    else
        goto R_FALSE;

R_FALSE :
    return FALSE;

R_TRUE :
    *pLm = lm; *pRm = rm;
    return TRUE;
}


void makePTypeCharts( SchartID, TchartID, lastTailV, pStackID )
    int SchartID, TchartID;
    int lastTailV;
    int *pStackID;
{
    int r;
    int PchartID;
    int headVStackID;
    long irrApplied;

    setFuncName( "makePTypeCharts" );

    irrUsed = FALSE;
    omittedUsed = FALSE;

    /* create stack to store the P-type chart's ID
     */
    r = isCreate();
    if( r < 0 )
        MAerror( END, E_MAXSTACK, __FILE__, __LINE__,
          "Max Stack is used now, please modify MAX_STACK value.\n" );
    *pStackID = r;

    /* create stack to store head vertex ID for one P-Type
     * chart which is used in recursive functions to
     * delete the previously check edges.
     */
    r = isCreate();
    if( r < 0 )
        MAerror( END, E_MAXSTACK, __FILE__, __LINE__,
          "Max Stack is used now, please modify MAX_STACK value.\n" );
    headVStackID = r;

    /* create one P-Type chart for recursive function
     */
    MAchartCreate( &PchartID );

    /* make P-Type charts for processing normal dictionary
     * search using dfs style recursive functions.
     */
    // isPush( headVStackID, 0 );
    irrApplied = 0x00000000;
    dfsMakePTypeChart( SchartID, TchartID, *pStackID,
                       PchartID, 0, lastTailV, headVStackID, &irrApplied );

    /* destroy stack and P-Type chart which is used in recursive
     * function
     */
    isDestroy( headVStackID );

    MAchartDestroy( PchartID );
    PchartID = -1;
    return;
}

static void dfsMakePTypeChart( SchartID, TchartID, stackID, PchartID, headV, lastTailV, hStackID, pIrrApplied )
    int SchartID, TchartID;
    int stackID;
    int PchartID;
    int headV;
    int lastTailV;
    int hStackID;
    long *pIrrApplied;
{
    register int i, j, k;
    int r, s;
    int IorE;
    int edgeID, e;
    int hv, tv;
    char rm, lm;
    char tStr[64], subStr[256], normalStr[256];

    setFuncName( "dfsMakePTypeChart" );

    /* base condition for recursive function
     * 1. make new P-Type chart and copy currently edges in chart
     * identified by PchartID to newly created chart
     * 2. and return.
     */
    if( headV == lastTailV ) {
        int tPchartID;

        MAchartCreate( &tPchartID );
        MAchartCopy( PchartID, tPchartID );
        MAchartPutIrrApplied( tPchartID, *pIrrApplied );
        isPush( stackID, tPchartID );
        return;
    }

    /* normal case must be handled firstly for the Unknown word processing.
     */
    r = MAchartGetByHeadV( SchartID, headV, &edgeID );
    if( r == FALSE )
        MAerror( END, E_STYPECHART, __FILE__, __LINE__,
          "S-type chart have no edge for headV %d.\n", headV );
    MAedgeHeadVOf( SchartID, edgeID, &hv );
    MAedgeTailVOf( SchartID, edgeID, &tv );
    MAedgeStrOf( SchartID, edgeID, tStr );

    MAchartInsert( PchartID, headV, tv, tStr, QQ, -1.0, &e );
    isPush( hStackID, headV );
    dfsMakePTypeChart( SchartID, TchartID, stackID, PchartID, tv, lastTailV, hStackID, pIrrApplied );
    // isPop() is in popAndDelete() function
    popAndDelete( hStackID, PchartID, headV );


    /* if there is some Irregular/Exceptional sub string which is started
     * from the current head vertex, then handle those firstly
     * and handle normal case next.
     */
    for( i=0; 1 ; i++ ) {

        r = MAchartGetIthByHeadV( TchartID, i, headV, &edgeID );
        if( r == FALSE ) break;
        MAedgeIorEOf( TchartID, edgeID, &IorE );
        MAedgeHeadVOf( TchartID, edgeID, &hv );
        MAedgeTailVOf( TchartID, edgeID, &tv );

        r = MAchartGetStringByInterval( SchartID, hv, tv, subStr );
        if( r == FALSE )
            MAerror( END, E_STYPECHART, __FILE__, __LINE__,
              "S-type chart is not matched to T-type chart.\n" );

        switch( IorE ) {
        case IEIrr :

            for( j=0; 1 ; j++ ) {
                int irrKind;
                int midV;

                /* in irrMakeEdges(), edges of which vertex ID is
                 * between hv and tv will be generated and insert to
                 * chart and according to that, head vertex stack will
                 * be updated.
                 */
                r = irrMakeEdges( PchartID, j, hv, tv, subStr, hStackID, &irrKind, &midV );
                if( r == FALSE ) break;

                /* omitted form */
                /* only one omitted form or irregular form can be applied
                 * to one word. So if some omitted form or irregular form
                 * is applied previously, then cancel to apply action
                 */
                if( irrKind & IRR_SHORT ) {
                    if( omittedUsed == TRUE ) {
                        popAndDelete( hStackID, PchartID, hv );
                        continue;
                    }
                    else
                        omittedUsed = TRUE;
                }
                else {
                    if( irrUsed == TRUE ) {
                        popAndDelete( hStackID, PchartID, hv );
                        continue;
                    } else
                        irrUsed = TRUE;
                }

                /* if this irregular is not occured always, then it must
                 * check Irregular dictionary and make chart
                 */
                if( irrKind & IRR_MASK ) {

                    for( k=0; 1 ; k++ ) {
                        int aPchartID; /* alternative PchartID */
    
                        /* after patten is matched, then some basic form must
                         * be in Irregular dictionary, if not then discard this
                         * one.
                         *
                         * if there is some matched basic form, then
                         * searchIdict() function makes new chart return TRUE.
                         */
                        s = searchIdict( PchartID, irrKind, midV, k, &aPchartID );
                        if( s == FALSE ) break;
    
                        *pIrrApplied |= irrKind;
                        dfsMakePTypeChart( SchartID, TchartID, stackID, aPchartID, tv, lastTailV, hStackID, pIrrApplied );
                        *pIrrApplied ^= irrKind;
                        MAchartDestroy( aPchartID );
                    }
                    // isPop() is in popAndDelete() function
                    popAndDelete( hStackID, PchartID, hv );

                }
                /* else it occured always, it did't need to check dictionary
                 */
                else {
                    *pIrrApplied |= irrKind;
                    dfsMakePTypeChart( SchartID, TchartID, stackID, PchartID, tv, lastTailV, hStackID, pIrrApplied );
                    *pIrrApplied ^= irrKind;
                    // isPop() is in popAndDelete() function
                    popAndDelete( hStackID, PchartID, hv );
                }

                /* resetting omittedUsed or irrUsed */
                if( irrKind & IRR_SHORT ) omittedUsed = FALSE;
                else irrUsed = FALSE;
            }
            break;

        case IEExce :

            MAedgeLmRmOf( TchartID, edgeID, &lm, &rm );

            for( j=0; 1 ; j++ ) {
                r = findStrEdict( subStr, &lm, &rm );
                /* if r is FALSE, then something is wrong */
                r = MAdictEGetIthData( EdictID, subStr, lm, rm, j, normalStr );
                if( r == FALSE ) break;

                /* normalStr is consisted of words and EPOS
                 * like irrMakeEdges() in exceMakeEdges(), chart and
                 * head vertex stack will be updated.
                 */
                exceMakeEdges( PchartID, hv, tv, normalStr, hStackID );

                *pIrrApplied |= IRR_EXCE;
                dfsMakePTypeChart( SchartID, TchartID, stackID, PchartID, tv, lastTailV, hStackID, pIrrApplied );
                *pIrrApplied ^= IRR_EXCE;
                // isPop() is in popAndDelete() function
                popAndDelete( hStackID, PchartID, hv );
            }
            break;

        default :
            MAerror( END, E_TTYPECHART, __FILE__, __LINE__,
              "T-type chart have inappropriate value.\n" );
            break;
        }
    } /* of switch */
}

/* if there is some basic form of which irregular type is irrKind,
 * then record that head vertex ID in pPreV, and make new chart
 * represented by pAPchartID, and return TRUE.
 */
static int searchIdict( PchartID, irrKind, midV, ith, pAPchartID )
    int PchartID;
    int irrKind;    /* 4 byte bit-vector */
    int midV;
    int ith;
#if 0
    char *dictStr;    /* IN-OUT : string to be searched in I-Type dict */
    int *pPreV;      /* IN-OUT : used for the first head vetex will be tested */
#endif
    int *pAPchartID; /* OUT : if return value is TRUE, then it value is useful */
{
    int r;
    int e, ee;
    int hv, tv;
    char dictStr[128], edgeStr[128];
    int irr;
    int eStack;

    strcpy( dictStr, "" );

    r = isCreate();
    if( r < 0 )
        MAerror( END, E_MAXSTACK, __FILE__, __LINE__,
          "Max Stack is used now, please modify MAX_STACK value.\n" );
    eStack = r;

    tv = midV;
    if( tv == 0 ) { isDestroy( eStack ); return FALSE; }

    do {
        MAchartGetByTailV( PchartID, tv, &e );
        MAedgeHeadVOf( PchartID, e, &hv );
        MAedgeStrOf( PchartID, e, edgeStr );

        /* if the edge contains already transformed datum(irregular or
         * exceptional word), then return FALSE immediately
         */
        {
            enum EPOS epos;
            MAedgeEPOSOf( PchartID, e, &epos );
            if( !(epos == QQ || epos == QP) ) {
                isDestroy( eStack );
                return FALSE;
            }
        }

        isPush( eStack, e );

        strcat( edgeStr, dictStr );
        strcpy( dictStr, edgeStr );

        r = MAdictIGet( IdictID, dictStr, &irr );
        if( r == TRUE && irr == irrKind ) {
            ith--;

            if( ith < 0 ) {

                /* if there is basic form of which irregular type is irrKind,
                 * then make the same chart, and change the edges including
                 * in basic form to only one edge cause it must be one word!
                 */
                MAchartCreate( pAPchartID );
                MAchartCopy( PchartID, *pAPchartID );
    
                while(1) {
                    r = isPop( eStack, &ee );
                    if( r == FALSE ) break;
    
                    MAchartDeleteByID( *pAPchartID, ee );
                }
    
                MAchartInsert( *pAPchartID, hv, midV, dictStr, QP, DEFAULT_FREQ, &ee );
    
                isDestroy( eStack );
                return TRUE;
            }
        }

        tv = hv;
    } while( tv != 0 );

    isDestroy( eStack );
    return FALSE;
}

static void exceMakeEdges( PchartID, headV, tailV, listOfWords, hStackID )
    int PchartID;
    int headV;
    int tailV;
    char *listOfWords;
    int hStackID;
{
    register int i;
    int r;
    char *p;
    int hv, tv, e;
    char *word, *eposStr;
    int tEpos;
    enum EPOS epos;
    int numOfSpace, numOfPair, idInterval;

    /* listOfWords : delimited by space character.
     * (word EPOS) pair will be appeared repeatedly.
     */

    delspace( listOfWords );

    /* count space of listOfWords, to know the number of
     * (word EPOS) pair.
     */
    for( i=0, numOfSpace = 0; i<strlen(listOfWords); i++ ) {
        if( listOfWords[i] == ' ' ) numOfSpace++;
    }

    numOfPair = numOfSpace/2 + 1;
    idInterval = (tailV - headV)/numOfPair;

    p = listOfWords;
    for( i=0; 1 ; i++ ) {
        word = (char*) strtok( p, " " ); p = NULL; if( word == NULL ) break;
        eposStr = (char*) strtok( NULL, " " );
        if( eposStr == NULL )
            MAerror( END, E_EDICTCONTENTS, __FILE__, __LINE__,
              "E-type dictionary's contents are wrong for word %s.\n", word);

        tEpos = str2epos( eposStr );
        if( tEpos == -1 )
            MAerror( END, E_EDICTEPOS, __FILE__, __LINE__,
              "EPOS value of E-type dictionary is wrong.\n");

        epos = (enum EPOS) tEpos;

        hv = headV + idInterval * i;
        tv = ( i == numOfPair-1 ) ? tailV : hv + idInterval;
        MAchartInsert( PchartID, hv, tv, word, epos, -1.0, &e );

        r = isPush( hStackID, hv );
        if( r == FALSE )
            MAerror( END, E_ISTACKPUSH, __FILE__, __LINE__,
              "Push op in integer stack is failed. full.\n" );
    }
    return;
}

static void popAndDelete( hStackID, PchartID, bottomV )
    int hStackID;
    int PchartID;
    int bottomV;
{
    int r;
    int headV;

    while(1) {
        r = isPop( hStackID, &headV );
        if( r == FALSE )
            MAerror( END, E_ISTACKPOP, __FILE__, __LINE__,
              "Pop op in integer stack is failed. empty.\n" );

        MAchartDeleteByHeadV( PchartID, headV );

        if( bottomV == headV ) break;
    }
}


/* return val :
 * 1 : if search processing is completed
 * 0 : if there is some unknown word, that is, searching is failed.
 */
int doNormalWordSearch( PchartID, RchartID, lastTailV )
    int PchartID;
    int RchartID;
    int lastTailV;
{
    register int i, j;
    int r, s, t;
    int isSuccessV;
    char subStr[256];
    float freq;
    int curV, headV;
    int sV;                 /* success vertex ID in ordered set */
    enum EPOS epos;
    int curEdgeID, e;

    setFuncName( "doNormalWordSearch" );

    /* clear and initialize the ordered set to store
     * the head vertex ID of result edges to be checked.
     */
    oCreate();

    /* first seed of ordered set is the last tail vertex ID
     */
    oInsert( lastTailV );

    /* for one syllable (edge) from end of word to start of word
     */
    isSuccessV = FALSE;
    curV = lastTailV;
    while(1) {

        r = MAchartGetByTailV( PchartID, curV, &curEdgeID );
        if( r == FALSE )
            MAerror( END, E_PTYPECHART_LC, __FILE__, __LINE__,
              "P-Type chart have lost chain, %d tail vertex ID.\n" );

        MAedgeEPOSOf( PchartID, curEdgeID, &epos );
        MAedgeHeadVOf( PchartID, curEdgeID, &headV );

        switch( epos ) {
        case QP :
            /* 1. if curV is not head vertex ID of result edge, fail.
             * 2. clear the ordered set except curV.
             * 3. do normal dictionary search.
             */
            if( !oExist( curV ) ) return FALSE;
            oDeleteBigValueThan( curV );
            r = doNS( PchartID, RchartID, headV, lastTailV );
            break;


        case QB :
            /* 1. do normal dictionary search.
             * 1.1 if normal dictionary searching step is succeded, that is,
             *     there exists some result edges of which head vertex ID
             *     is headV, then clear the ordered set except headV.
             * 1.2 if not, fail.
             */
            r = doNS( PchartID, RchartID, headV, lastTailV );
            if( r == FALSE ) return FALSE;
            oDeleteBigValueThan( headV );
            break;

        case QQ :
            /* normal processing.
             */
            r = doNS( PchartID, RchartID, headV, lastTailV );
            break;

        default :
            /* 1. only check the connectivity btn. the current edge and
             *   result edge of which head vertex ID is curV.
             * 1.1 if exist, clear ordered set and put only headV in ordered
             *    set.
             * 1.2 if not, fail.
             */
            if( curV == lastTailV ) {
                if( cTable[epos][FIN] == '1' ) {
                    MAedgeStrOf( PchartID, curEdgeID, subStr );
                    MAedgeFreqOf( PchartID, curEdgeID, &freq );

                    MAchartInsert( RchartID, headV, curV, subStr, epos, freq, &e );
                    oInsert( headV );
                    oDeleteBigValueThan( headV );
                }
                else return FALSE;
            } else {
                for( t=TRUE, j=0; t ; j++ ) {
                    int edgeID;
                    enum EPOS nextEpos;

                    t = MAchartGetIthByHeadV( RchartID, j, curV, &edgeID );
                    if( t == FALSE ) break;
                    MAedgeEPOSOf( RchartID, edgeID, &nextEpos );

                    if( cTable[epos][nextEpos] == '1' ) {
                        MAedgeStrOf( PchartID, curEdgeID, subStr );
                        MAedgeFreqOf( PchartID, curEdgeID, &freq );

                        MAchartInsert( RchartID, headV, curV, subStr, epos, freq, &e );
                        MAchartConnectEdges( RchartID, e, edgeID );
                        oInsert( headV );
                        oDeleteBigValueThan( headV );
                        isSuccessV = TRUE;
                    }
                }
                if( isSuccessV != TRUE ) return FALSE;
            }
            r = TRUE;

        } // of switch

        /* in the last, head vertex will be the first one, then
         * break this loop.
         */
        if( headV == 0 ) break;
        else {
            /* step forward */
            MAedgeHeadVOf( PchartID, curEdgeID, &headV );
            curV = headV;
        }
    } /* for one syllable */

    /* why this value 'r' represent the TRUE/FALSE of this
     * functions is a little bit clumsy, so please check the
     * codes in switch statements (QQ, QP, QB, default)
     * carefully when that statements are executed at
     * the first vertex head ID, then you can get the answer
     */
    return r;
}

/* this routine must be called from doNormalSearch().
 * return TRUE if headV is inserted in ordered set.
 * otherwise return FALSE.
 */
static int doNS( PcID, RcID, headV, lastTailV )
    int PcID, RcID;
    int headV;
    int lastTailV;
{
    register int i,j;
    int r, s, t;
    char string[256];
    int sV;
    int isSuccessV;
    int isInserted;
    int edgeID, e;
    enum POS pos;
    float freq;

    /*
     * sV : a vertex in ordered set, that is, the head vertex ID of
     *      result edges.
     *
     *           string
     * headV o ---------- o sV
     *
     * 1. get the 'string' btn. headV and sV.
     * 2. for the words in normal dictionary of which keyword is 'string',
     *    - check the connetivity of current word(edge) and the result
     *      edge of which head vertex ID is sV. If can be connected, then
     *      insert 'string'(edge) to R-Type chart and connect two.
     */

    isSuccessV = FALSE;
    for( r = oGetFirst( &sV ); r != O_NO_DATA; r = oGetNext( &sV ) ) {

        /* get the 'string' btn. headV and sV
         */
        s = MAchartGetStringByInterval( PcID, headV, sV, string );
        if( s == FALSE )
            MAerror( END, E_INTERNAL, __FILE__, __LINE__, "P-type chart have lost chain.\n" );

        /* for the words in normal dictionry
         */
        for( s = TRUE, i=0; s ; i++ ) {

            /* s = MAdictNGetIthPOS( NdictID, string, i, &pos, &freq ); */
            s = cacheFind( NdictID, string, i, &pos, &freq );
            if( s != TRUE ) break;

            /* check the connectivity and if there is some connectivity
             * then insert it to R-Type chart and connect.
             */
            if( sV == lastTailV ) {
                if( cTable[pos][FIN] == '1' ) {
                    MAchartInsert( RcID, headV, sV, string, pos, freq, &e );
                    isSuccessV = TRUE;
                }
            } 
            else {
                /* for one string in normal dictionary, one one edge
                 * can be made, but several connection can existed
                 * between inserted edges and the existed result edges.
                 */
                isInserted = FALSE;

                for( t=TRUE, j=0; t ; j++ ) {
                    enum POS nextPos;

                    t = MAchartGetIthByHeadV( RcID, j, sV, &edgeID );
                    if( t == FALSE ) break;

                    MAedgePOSOf( RcID, edgeID, &nextPos );

                    if( cTable[pos][nextPos] == '1' ) {
                        if( isInserted == FALSE ) {
                            MAchartInsert( RcID, headV, sV, string, pos, freq, &e );
                            isInserted = TRUE;

                            /* 'e' variable contains currently inserted
                             * edge's ID continuously.
                             */
                        }
                        MAchartConnectEdges( RcID, e, edgeID );
                        isSuccessV = TRUE;
                        // oInsert( headV );
                    }
                }
            }
        } /* for the word in normal dictionary */
    }

    /* the routine to update ordered set must be here, cause
     * if insertion occur in the middle of 'for' loop, then
     * that next call to oGetNext() routine can return wrong
     * value.
     */
    if( isSuccessV == TRUE ) {
        oInsert( headV );
        if( s < 0 )
            MAerror( END, E_ORDERED + s, __FILE__, __LINE__, oErrorStr );
        return TRUE;
    }
    else return FALSE;
}

void doPrintResult( RchartID, lastTailV, irrApplied, resultKind )
    int RchartID;
    int lastTailV;
    long irrApplied;
    char resultKind;
{
    register int i;
    int r;
    int edgeID;
    int tailV;
    int stackID;

#if !defined(KTS)
    printf("--- RESULT BEGIN\n" );
#endif
    r = isCreate();
    if( r < 0 )
        MAerror( END, E_MAXSTACK, __FILE__, __LINE__,
          "Max Stack is used now, please modify MAX_STACK value.\n" );
    stackID = r;

#if !defined(KTS)
    printf("Applied :");
    printIrrApplied( irrApplied );
#endif

    for( r = TRUE, i=0; r ; i++ ) {
        r = MAchartGetIthByHeadV( RchartID, i, 0, &edgeID );
        if( r == FALSE ) break;

        MAedgeTailVOf( RchartID, edgeID, &tailV );

        isPush( stackID, edgeID );
        dfsDoPrintResult( stackID, RchartID, edgeID, irrApplied, resultKind, lastTailV );
        isPop( stackID, &edgeID );
    }

    isDestroy( stackID );
#if !defined(KTS)
    printf("--- RESULT END\n" );
#endif
    return;
}

static void dfsDoPrintResult( stackID, RchartID, edgeID, irrApplied, resultKind, lastTailV )
    int stackID;
    int RchartID;
    int edgeID;
    long irrApplied;
    char resultKind;
    int lastTailV;
{
    register int i;
    int r;
    int nextEdgeID, e;
    int tailV;
    char str[128], wordKS[128];
    enum POS pos;
#if defined(DEBUG) || defined(DEBUGR)
    float freq;
    int hv, tv;
#endif

    /* the last condition :
     * see the contents of stack and print the result
     */
    MAedgeTailVOf( RchartID, edgeID, &tailV );

    if( tailV == lastTailV ) {
#if !defined(KTS)
        printf("--- --- BEGIN\n");
        printf("         eID   hv   tv   pos  frq  word\n");
#endif

        for( r=TRUE, i=0; r ; i++ ) {
            r = isGetIth( stackID, i, &e );
            if( r == FALSE ) break;
            MAedgeStrOf( RchartID, e, str );
            MAdictNKey2KS( str, wordKS );
            MAedgePOSOf( RchartID, e, &pos );
#if defined(DEBUG) || defined(DEBUGR)
            MAedgeFreqOf( RchartID, e, &freq );
            MAedgeHeadVOf( RchartID, e, &hv );
            MAedgeTailVOf( RchartID, e, &tv );

            printf("        %4d  %3d  %3d  %4s  %f  %s\n", e, hv, tv, POSstr[pos], freq, wordKS );
#else
#if !defined(DEBUG) && !defined(DEBUGR) && defined(KTS)
            if( i==0 )
                printf("%c %s/%s", resultKind, wordKS, POSstr[pos] );
            else
                printf("+%s/%s", wordKS, POSstr[pos] );
#else
            /* this routine must be changed to accomodate the
             * the other's request about output format.
             */
            printf("        %4d\n", e );
#endif
#endif
        }
#if !defined(KTS)
        printf("--- --- END\n");
#else
        printf(" ");
        printIrrApplied( irrApplied );
#endif
        return;
    }

    for( r=TRUE, i=0; r ; i++ ) {
        r = MAedgeIthNextOf( RchartID, edgeID, i, &nextEdgeID );
        if( r == FALSE ) break;

        isPush( stackID, nextEdgeID );
        dfsDoPrintResult( stackID, RchartID, nextEdgeID, irrApplied, resultKind, lastTailV );
        isPop( stackID, &nextEdgeID );
    }
    return;
}

static void printIrrApplied( iA )
    long iA;
{
    register int i;

    for( i=0; i<32; i++ ) {
        if( iA & 0x80000000 ) printf(" %s", irrStr[i] );

        iA <<= 1;
    }

    printf("\n");
}


/* this routine is very similiar to doPrintResult() in dfs-style
 * approach except that doPrintResult() routine search the result
 * edges from the first edge to the last edge but this routines do
 * from the last edges to the first edges.
 */
void doUnknownWordProcessing( PchartID, RchartID, lastTailV )
    int PchartID;
    int RchartID;
    int lastTailV;
{
    register int i;
    int r;
    int edgeID, e;
    int headV;
    int stackID;
    int isProcessed = FALSE;
    char edgeStr[128];

#if defined(DEBUG)
    printf("--- UNKNOWN WORD PROCESSING BEGIN\n" );
#endif
    r = isCreate();
    if( r < 0 )
        MAerror( END, E_MAXSTACK, __FILE__, __LINE__,
          "Max Stack is used now, please modify MAX_STACK value.\n" );
    stackID = r;

    for( r = TRUE, i=0; r ; i++ ) {
        r = MAchartGetIthByTailV( RchartID, i, lastTailV, &edgeID );
        if( r == FALSE ) break;

        MAedgeHeadVOf( RchartID, edgeID, &headV );

        isPush( stackID, edgeID );
        r = dfsDoUWP( PchartID, RchartID, edgeID, stackID /*, lastTailV */);
        if( r == TRUE ) isProcessed = TRUE;
        isPop( stackID, &edgeID );
    }
    /* After this processing, if there is no handling about the
     * Unknown word results, then handle the whole word.
     */
    if( isProcessed == FALSE ) {
        MAchartGetStringByInterval( PchartID, 0, lastTailV, edgeStr );
        for( i = 0; i < NUM_POS; i++ ) {
            if( mTable[INI][i] == '1' && mTable[i][FIN] == '1' ) {
                MAchartInsert( RchartID, 0, lastTailV, edgeStr, i, 0.0, &e );
                /* MAchartConnectEdges( RchartID, e, edgeID ); */
            }
        }
    }

    isDestroy( stackID );
#if defined(DEBUG)
    printf("--- UNKNOWN WORD PROCESSING END\n" );
#endif
    return;
}

/* return val :
 * 1 : if at least one unknown word is processed, and can print
 *     the results to user.
 * 0 : otherwise
 */
static int dfsDoUWP( PchartID, RchartID, edgeID, stackID )
    int PchartID;
    int RchartID;
    int edgeID;
    int stackID;
{
    register int i, j;
    int r;
    int prevEdgeID, e;
    int headV;
    char edgeStr[128];
    enum POS pos;
    enum EPOS epos;
    int h;
    int isInUdict = FALSE;
    int isProcessed = FALSE;

    for( r=TRUE, i=0; r ; i++ ) {
        r = MAedgeIthPrevOf( RchartID, edgeID, i, &prevEdgeID );
        if( r == FALSE ) break;

        isPush( stackID, prevEdgeID );
        isProcessed = dfsDoUWP( PchartID, RchartID, prevEdgeID, stackID /*, lastTailV */);
        isPop( stackID, &prevEdgeID );
    }
    /* if there is no previous edge of current edge,
     * 1. check the current edge's contents(strings) are in
     *    unknown word dictionary.
     * 2. if there is one, then use retrieved informaton to
     *    guess unknown POS.
     * 3. if there in none, use mTable to guess unknown POS
     */
    if( i == 0 ) {
        /* check the current edge's contents are in unknown
         * word dictionary
         * if there is one, then update R-Type chart
         */
        MAedgeStrOf( RchartID, edgeID, edgeStr );
        MAedgePOSOf( RchartID, edgeID, &epos );
        for( h = TRUE, j=0; h == TRUE; i++ ) {
            h = MAdictUGetIthPOS( UdictID, edgeStr, epos, j, &pos );
            if( h == TRUE ) {
#if defined(DEBUG) || defined(DEBUGU)
                printf("    %s %s is in U-Type dictionary - checked.\n", edgeStr, EPOSstr[epos] );
#endif
                MAedgeHeadVOf( RchartID, edgeID, &headV );
                MAchartGetStringByInterval( PchartID, 0, headV, edgeStr );
                MAchartInsert( RchartID, 0, headV, edgeStr, pos, 0.0, &e );
                MAchartConnectEdges( RchartID, e, edgeID );
                isInUdict = TRUE;
                isProcessed = TRUE;
            }
        }

        /* use mTable or retrieved information to guess
         * unknows POS, and modify the values in stack appropriately.
         */
        if( isInUdict == FALSE ) {
#if defined(DEBUG) || defined(DEBUGU)
            printf("    mTable is used for unknown word guessing : ");
#endif
            pos = (enum POS) epos;
            for( i=0; i<NUM_POS; i++ ) {
                if( mTable[i][pos] == '1' && mTable[INI][i] == '1' ) {
#if defined(DEBUG) || defined(DEBUGU)
                    printf("+");
#endif
                    MAedgeHeadVOf( RchartID, edgeID, &headV );
                    MAchartGetStringByInterval( PchartID, 0, headV, edgeStr );
                    MAchartInsert( RchartID, 0, headV, edgeStr, i, 0.0, &e );
                    MAchartConnectEdges( RchartID, e, edgeID );
                    isProcessed = TRUE;
                }
            }
#if defined(DEBUG) || defined(DEBUGU)
            printf("\n");
#endif
        }
    }
    return isProcessed;
}
