/* header for MAerror.h
 * all functions which use error function must include this header file.
 *
 * create : 95.5.11 (Thu), bgjang@csone - firstly written
 * modify :
 * 95.5.29 - 95.6.1 (Thu), bgjang@csone - error code is inserted frequently
 *
 */
#ifndef __MAERROR_H__
#define __MAERROR_H__

/* definitions --------------------------------------------------- */
#define L_FUNCNAME	16

/* severity : must be 0,1 (related to codes in MAerror() func.) */
#define END		0	/* exit program */
#define CON		1	/* continue, only warning message */
#define NOT		2	/* notify, continue */


/* extern variables ---------------------------------------------- */
extern char functionName[];	/* functionName must be set. */


/* function declarations ----------------------------------------- */
extern void setFuncName(/* funcName */);
extern void MAerror(/* severity, errorcode, fileName, lineNo,
		       fmt, var1, var2, ... */);
#ifdef DEBUG
extern void debugMesg(/* fmt, var1, var2, ... */);
#else
extern void debugMesg();
#endif

#define MAerrorInternal(X) MAerror(END,(X),__FILE__,__NAME__,"Internal error.\n")


/* definition of all error/warning/notify code ------------------- */
/* all definition's value must be negative except E_NOERROR, E_ERROR */
#define E_NOERROR	1	/* no error */
#define E_ERROR		0	/* error */
#define E_MALLOC        (-1)     /* malloc error */
#define E_REALLOC       (-2)     /* realloc error */
#define E_INTERNAL      (-3)     /* programming code is wrong */
#define E_FILEOPEN      (-4)     /* file open is failed */
#define E_OPENCTABLE    (-5)     /* connectivity table file open failed */
#define E_WRONGCTABLE   (-6)     /* wrong format connectivity table */
#define E_OPENMTABLE    (-7)     /* mtable file open failed */
#define E_EDICTOPEN     (-9)     /* Exception dictionary is not opened. */
#define E_NDICTOPEN     (-10)    /* Normal dictionary is not opened. */
#define E_IDICTOPEN     (-11)    /* Irregular dictionary is not opened. */
#define E_UDICTOPEN     (-12)    /* Unknown word dictionary is not opened. */
#define E_ENVVAR        (-14)    /* environment variables didn't be set */
#define E_TTYPECHART    (-19)    /* error in T-type chart in MoA */
#define E_STYPECHART    (-20)    /* error in S-type chart in MoA */
#define E_MAXSTACK      (-21)    /* Max Stack is used now */
#define E_EDICTCONTENTS (-22)    /* Contents of E-type dictionary is wrong */
#define E_EDICTEPOS     (-23)    /* EPOS value of E-type dictionary is wrong */
#define E_ISTACKPUSH    (-40)    /* integer stack Push error - full of stack */
#define E_ISTACKPOP     (-41)    /* integer stack Pop error - empty of stack */
#define E_PTYPECHART_LC (-50)    /* P-type have lost chain */

/* ordered set */
#define E_ORDERED       (-750)  /* from E_ORDERED, ordered set routines */
                                /* error will be counted. */

/* chart */
#define E_CHART_TYPE    (-800)  /* wrong chart type in argument */
#define E_CHART_ID      (-801)  /* chart id is wrong */
#define E_CHART         (-850)  /* from E_CHART, chart routines' error
                                 * will be counted, so error code is
                                 * E_CHART + <private error code from -1>
                                 */
#if 0
#define E_EDGEPOOL      (-850)  /* from E_EDGEPOOL, edge pool routines' */
                                /* error will be count, so error code is */
                                /* E_EDGEPOOL + <private error code from -1> */
#endif

// dictionary
#define E_DICT_TYPE     (-900)  /* wrong type in argument */
#define E_NO_POS        (-901)  /* no proper POS to be processed */
#define E_BIG_ITH       (-902)  /* ith value is too big */
#define E_NO_WORD       (-903)  /* word is not existed in database */
#define E_EXIST_POS     (-904)  /* POS is existed already */
#define E_ETYPE_DATA    (-905)  /* e-type dictionary's data have wrong form */
#define E_POS           (-906)  /* wrong POS value in input */
#define E_EPOS          (-907)  /* wrong EPOS value in input */
#define E_EXIST_DATA    (-908)  /* Data is existed already */
#define E_DICT          (-950)  /* error code for dictionary API starting */
                                /* from E_DICT, so error code is */
                                /* E_DICT + <private error code from -1> */

#endif
