/* cache for normal dictionary
 *
 * create : 95.7.21 (Fri), bgjang
 * it takes more than one day to decide the structure of cache
 * for normal dictionary, it's structure is modified from the
 * hashing technique using chain stretagy.
 * modify :
 *
 *
 */
#include <stdio.h>
#include "chart.h"   /* for MAX_WORD_LEN only */
#include "pos.h"
#include "common.h"
#include "version.h"

#if defined(DEBUG)
static void cacheDumpOneElts();
#endif

/* log file of caching
 * if don't want to log the usage of log, then it must be commented out
 * or undef LOG_STATISTIC when compiling.
 */
#if defined(LOG_STATISTIC)
#define CACHE_LOG   "log/cache_usage"
#endif

/* MAX_SAME_HASHVAL elements can have same hash values.
 * So, it can be thought as there is limit value for one chain 
 * in Hashing. It's value must be choosen for the sequentail search time
 * to be the best among several sorting methods.
 * And There is MAX_CACHE_HASHTABLE_BUCKET hashing buckets are existed.
 * It must be selected as prime number.
 */
#define MAX_SAME_HASHVAL             7
#define MAX_CACHE_HASHTABLE_BUCKET   503

#define CACHE_HASHVAL(k)    ((k)%MAX_CACHE_HASHTABLE_BUCKET)

struct aBucket {
    char word[MAX_WORD_LEN];
    float freq[NUM_POS];    /* POS vlaue is used as index of this array */
    int lookup;             /* the freq of lookup - to select victim */
};

/* a memory space used as Hash */
static struct aBucket hashTable[MAX_CACHE_HASHTABLE_BUCKET][MAX_SAME_HASHVAL];

#if defined(CACHE_LOG)
static int hit = 0;
static int nohit = 0;
#endif


void cacheInit()
{
    register int i,j;

    for( i=0; i<MAX_CACHE_HASHTABLE_BUCKET; i++ )
        for( j=0; j<MAX_SAME_HASHVAL; j++ )
            hashTable[i][j].lookup = 0;
}


/* this function is used instead of MAdictNGetIthPOS()
 * so the argument meanings are same with those of MAdictNGetIthPOS()
 *
 * modify? if string is NULL, then use the previous words?
 *         if i is -1, then get the next POS value ?
 */
int cacheFind( dictID, word, ith, pPos, pFreq )
    int dictID;
    char *word;
    int ith;
    enum POS *pPos;
    float    *pFreq;
{
#define MAX_LOOK 7
    register int i, j;
    int r;
    int N, value;
    int victimIndex;

    if( isUsingCache == FALSE )
        return MAdictNGetIthPOS( dictID, word, ith, pPos, pFreq );


    /* word to integer : this routine is same with _hashStr2Integer(),
     * so must be shared, but ...
     */
    N = MIN( strlen(word), MAX_LOOK );
    for( value=0, i=0; i<N; i++ )
        value += (unsigned char) word[i];

    value = CACHE_HASHVAL(value);  /* hash value used as index */

    /* find the victim in the MAX_SAME_HASHVAL elements of which
     * hash valus is same. Notice that the whole elements are not
     * considered to take a victim!
     *
     * and concurrently searching word.
     */
    for( i=0, victimIndex = 0; i<MAX_SAME_HASHVAL; i++ ) {
        if( hashTable[value][i].lookup == 0 ) {
            victimIndex = i;
            break;
        }
        if( !strcmp( hashTable[value][i].word, word ) ) {
            /* found - find ith POS, if there, then return TRUE */
#if defined(CACHE_LOG)
            hit++;
#endif
            for( j=0; j<NUM_POS; j++ ) {
                if( hashTable[value][i].freq[j] != 0 ) ith--;
                if( ith < 0 ) {
                    *pPos = (enum POS) j;
                    *pFreq = hashTable[value][i].freq[j];
                    return TRUE;
                }
            }
            return FALSE;
        }
        if( hashTable[value][victimIndex].lookup > hashTable[value][i].lookup )
            victimIndex = i;
    }

#if defined(CACHE_LOG)
    nohit++;
#endif

    /* if there is none, then read from the dictionary and store it
     * in cache and return one.
     */
    r = MAdictNGetPOSArray( dictID, word, hashTable[value][victimIndex].freq );

    if( r == FALSE ) return FALSE;
    else {
        /* store new element in hash table */
        strcpy( hashTable[value][victimIndex].word, word );
        hashTable[value][victimIndex].lookup = 1;

        for( j=0; j<NUM_POS; j++ ) {
            if( hashTable[value][victimIndex].freq[j] != 0 ) ith--;
            if( ith < 0 ) {
                *pPos = (enum POS) j;
                *pFreq = hashTable[value][victimIndex].freq[j];
                return TRUE;
            }
        }
        return FALSE;
    }
}

extern void cacheDestroy()
{
#if defined(CACHE_LOG)
    FILE *fp;
    char s[100];

    fp = fopen( CACHE_LOG, "a+" );

    timeStr( s );
    fprintf( fp, "%s - ", s);
    fprintf( fp, "%s (%s) - ", VERSION, REVDATE );
    fprintf( fp, "%d/(%d + %d) = %f hit ratio\n", hit, hit, nohit, (float) hit / (float) (hit+nohit) );
    fclose(fp);
#endif

    /* it is not bad to store cache image in some file, and use
     * this in the next execution time
     */
    ;
}

#if defined(DEBUG)
void cacheDump()
{
    register int i, j;
    int nl;

    printf("--- Cache Dump BEGIN\n");
    for( i=0; i<MAX_CACHE_HASHTABLE_BUCKET; i++ ) {
        for( nl=0, j=0; j<MAX_SAME_HASHVAL; j++ ) {
            if( hashTable[i][j].lookup != 0 ) {
                nl = 1;
                cacheDumpOneElts( &(hashTable[i][j]) );
            }
        }
        if( nl == 1 ) printf("%dth\n", i);
    }
    printf("--- Cache Dump END\n");
}

static void cacheDumpOneElts( p  )
    struct aBucket *p;  /* pointer to a bucket */
{
    register int i;

    printf("%s ", p->word );
    for( i=0; i<NUM_POS; i++ )
        if( p->freq[i] != 0 )
            printf("%s ", POSstr[i]);
    printf(":");
}
#endif
