/* header file for chart.c
 *
 * create : 95.6.24 (Sat), bgjang@csone
 * modify :
 *
 */
#ifndef __CHART_H__
#define __CHART_H__

#include "pos.h"


/* config values ------------------------------------------------------*/

/* for one word, the value 40 is expected the maximum value.
 * the followings are the chart type(?) list which is used by MAK.
 *
 * 1. the chart for input itself
 * 2. the chart to store the result to check irregular/exception
 *   based on first chart
 * 3. the several charts generated by irregular/exception processing
 *   which can be used by the last normal word searching step.
 * 4. the chart to store the result of normal word searching step.
 */
#ifndef MAX_CHART
#define MAX_CHART   100
#endif

/* maximum edges in one chart.
 *
 * One Hangul characters(composed by 2-bytes in KS code)is consisted of
 * about 2 or 3 syllable. So, the maximum word length limited by this
 * max-value is about 60 characters (really big one, I guess).
 */
#ifndef MAX_EDGES_IN_CHART
#define MAX_EDGES_IN_CHART    200
#endif

/* maximum buckets in one Hash table
 *
 * This value is concerned to the following fixed values.
 * 1. VERTEX_ID_INTERVAL - Vertex's ID is firstly interleaved by the
 *   value of VII. So I think that for the value VII, 7 is proper!
 * 2. MAX_EDGES_IN_CHART - I expect that for the number of elts,
 *   150% of the number of edges is proper
 * 3. Prime number - must be prime number to use double hash
 */
#ifndef MAX_BUCKETS
#define MAX_BUCKETS   311
#endif

/* maximum length of one word
 *
 * in data for one edge, there is word. and this length is maximized
 * when chart is used to store the result of one word in normal
 * dictionary or unknown word.
 *
 * Hangul datum is stored in Kimmo code. So one character is about
 * 3 byte. So about 20 Hangul characters can be stored, if MAX_WORD_LEN
 * is 60.
 */
#ifndef MAX_WORD_LEN
#define MAX_WORD_LEN   60
#endif


/* maximum path from one edge to the other edge
 *
 * must be thought.
 */
#ifndef MAX_PATH
#define MAX_PATH       100
#endif


/* data type for one edge -------------------------------------------*/

struct edgeData {
    char      str[MAX_WORD_LEN];
    enum EPOS epos;
    float     freq;
};

struct edge {
    /* int id; */
    /* id is represented implicitly as index of array */
    int headV;
    int tailV;
    struct edgeData d;

    short  prevEdges[MAX_PATH];    /* list of edge ID */
    short  nextEdges[MAX_PATH];
};


/* declarations of global variable -------------------------------*/

extern char chartES[];


/* function declarations -----------------------------------------*/

extern void chartInit();
extern int  chartCreate(/* pId */);
extern int  chartDestroy(/* id */);
extern int  chartInsertEdge(/* id, headV, tailV, str, epos, freq, pEdgeID */);
extern int  chartConnect(/* id, edgeIDA, edgeIDB */);
extern int chartPutIrrApplied(/* id, irrApplied */);
extern int chartGetIrrApplied(/* id, pIrrApplied */);
extern int  chartGetIth(/* id, ith, pEdgeID */);
extern int  chartGetIthByHeadV(/* id, ith, headV, pEdgeID */);
extern int  chartGetIthByTailV(/* id, ith, tailV, pEdgeID */);
extern int  chartGetIthByData(/* id, ith, str, pEdgeID */);
extern int  chartGet(/* id, pEdgeID, first */);
extern int  chartDelete(/* id, edgeID */);
extern int  chartCopy(/* fromID, toID */);
#if defined(DEBUG)
extern void chartDump(/* id, isHH, isHT, isHE */);
#endif

extern int  edgeStrOf(/* id, edgeID, str */);
extern int  edgeEPOSOf(/* id, edgeID, pEpos */);
extern int  edgePOSOf(/* id, edgeID, pPos */);
extern int  edgeFreqOf(/* id, edgeID, pFreq */);
extern int  edgeIorEOf(/* id, edgeID, pIorE */);
extern int  edgeHeadVOf(/* id, edgeID, pHeadV */);
extern int  edgeTailVOf(/* id, edgeID, pTailV */);
extern int  edgeIthNextOf(/* id, edgeID, ith, pNext */);
extern int  edgeIthPrevOf(/* id, edgeID, ith, pPrev */);


/* error code ---------------------------------------------*/

#define C_DUP             3    /* duplicate */
#define C_NO_DATA         2    /* not found */
#define C_ERR_NO          1
#define C_ERR_MAX_CHART   -1   /* Max Chart is used now */
#define C_ERR_CHART_ID    -2   /* chart ID is not proper one */
#define C_ERR_MAX_EDGES   -3   /* Max Edges in chart is used */
#define C_ERR_HASH_FULL   -4   /* Hash Table is full */
#define C_ERR_EDGE_ID     -5   /* edge ID is not proper one */
#define C_ERR_HEADV       -6   /* error in head vertex ID */
#define C_ERR_TAILV       -7   /* error in tail vertex ID */
#define C_ERR_MAX_PATH    -8   /* path data is full */
#define C_ERR_EMPTY_CHART -9   /* chart ID is for empty chart */


#endif
