/*----------------------------------------------------------*
 *                                                          *
 * Korean Part-Of-Speech Tagging System ( kts ) Version 0.9 *
 *                                                          *
 * Sang Ho Lee                                              *
 *                                                          *
 * jhlee@csone.kaist.ac.kr shlee2@adam.kaist.ac.kr          *
 *                                                          *
 * Computer Systems Lab.                                    *
 * Computer Science , KAIST                                 *
 *                                                          *
 * irrtbl.h ( Korean POS Tagging System : irr table )       *
 *                                                          *
 *----------------------------------------------------------*/

#define __NUMIRR__ 20 
#define _IRR1_     4
#define _IRR2_     113
#define _IRR3_     187
#define _IRR4_     29
#define _IRR5_     10

typedef struct _irr_ {
   char pattern[6]    ; /* ұĢ Ͼ pattern        */
   int  irrhandle[3]  ; /* ұĢ code                      */
} IRR ; 

typedef struct _irtbl_ {
   IRR *irr_tbl       ; /* ұĢ table  Ű pointer */
   int size           ; /* table ũ                     */ 
} IRRTABLE ;  

int irrShift[__NUMIRR__] = {
           -1 , /* 0 */
	   0  , /* 1 */
	   -1 , /* 2 */
	   -1 , /* 3 */
	   0  , /* 4 */
	   -2 , /* 5 */
	   -2 , /* 6 */
	   -1 , /* 7 */
	   -2 , /* 8 */
	   -2 , /* 9 'ְŶ' ''     pos */  
	   -4 , /* 10 */
	   -1 , /* 11 */
	   0  , /* 12 */
	   0  , /* 13 */
	   0  , /* 14 */
	   -1 , /* 15 += (ü ,    ) */
	   -1 , /* 16 += , += (  )  */
	   -1 , /* 17 += , += (  )  */
	   0  , /* 18 += */
	   -1 , /* 19 '' Ż  */
   } ; 

/* 0 : '' Ż : trigger character ٷ ʿ '_'            
					 ( ʼ ,   , V() ; ȴ)        */
/* 1 : '' Ż : trigger character ٷ ʿ 'L'  : 
			   ʿ { N | n | B | si | o | L | s9 }  ϳ
			   ʿ ʼ  ȵ :   Ȥ ߼ ´       */
/* 2 : '' ұĢ : trigger character ٷ ʿ 'S'              */ 
/* 3 : '' ұĢ : trigger character ٷ  character 'D' ٲ */
/* 4 : '' ұĢ : trigger character 'B' ٲ                     */
/* 5 : '' ұĢ : trigger character ٷ  character '_' ٲ */
/* 6 : '' ұĢ : trigger character ٷ  character        */
/* 7 : '' ұĢ : trigger character ٷ ʿ 'u'              */
/* 8 : '' ұĢ : trigger character ٷ  character        */
/* 9 : 'Ŷ'ұĢ: trigger character ι°  character      */
/* 10: 'ʶ'ұĢ: trigger character °  character      */
/* 11: '' ұĢ : trigger character ٷ ʿ 'H'              */
/* 12: '' ұĢ : trigger character 'aHa'/ 'eHe' ϳ 
					   缺̳ ̳Ŀ              */
/* 13: ',' : trigger characterڿ 'e'  , /ڸ   
					  ''   ϳ. (   ȴ)*/
/* 14: ''    : trigger character 'ae' ٲ                      */
/* 15: '+='  : trigger character  'y' 'i' ٲ  (ü)  */
/* 15: '+='  : trigger character  'y' 'i' ٲ  ()
					 ( ʼ ,   , V() ; ȴ)        */
/* 16: '+='  : trigger character  'w' 'o' ٲ 
					 ( ʼ ,   , V() ; ȴ)        */
/* 16: '+='  : trigger character  'w' 'u' ٲ 
					 ( ʼ ,   , V() ; ȴ)        */
/* 17: '+='  : trigger character ʿ 'a'  ( )
					 ( ʼ ,   , V() ; ȴ)        */
/* 17: '+='  : trigger character ʿ 'e'  ( ) 
                     ( ʼ ,   , V() ; ȴ)        */
/* 18: '+='  : trigger character 'ie' ٲ (   ٲ) 
                     ( ʼ ,   , V() ; ȴ)        */ 
/* 19: ' Ż'   : trigger character ι° ʿ 'i'            */ 

IRR irr_tbl1[_IRR1_] = {
     { "_" , { 1 , -1 , -1 } } , 
     { "a" , { 1 , -1 , -1 } } , 
     { "i" , { 1 , -1 , -1 } } , 
     { "u" , { 1 , -1 , -1 } }
  } ;

IRR irr_tbl2[_IRR2_] = {
     { "S8" , { 13 , -1 , -1 } } , 
     { "b8" , { 13 , -1 , -1 } } , 
     { "b9" , { 13 , -1 , -1 } } , 
     { "be" , { 1 , -1 , -1 } } , 
     { "bi" , { 1 , -1 , -1 } } , 
     { "bu" , { 1 , -1 , -1 } } , 
     { "c8" , { 13 , -1 , -1 } } , 
     { "ca" , { 17 , -1 , -1 } } , 
     { "ci" , { 1 , -1 , -1 } } , 
     { "d8" , { 12 , 13 , -1 } } , 
     { "d9" , { 13 , 19 , -1 } } , 
     { "d_" , { 1 , 19 , -1 } } , 
     { "da" , { 1 , 19 , -1 } } , 
     { "de" , { 1 , 19 , -1 } } , 
     { "di" , { 19 , -1 , -1 } } , 
     { "do" , { 1 , 19 , -1 } } , 
     { "dw" , { 19 , -1 , -1 } } ,
     { "du" , { 1 , -1 , -1 } } , 
     { "eu" , { 4 , -1 , -1 } } , 
     { "ew" , { 4 , -1 , -1 } } , 
     { "f8" , { 12 , 13 , -1 } } , 
     { "f9" , { 13 , -1 , -1 } } , 
     { "f_" , { 1 , -1 , -1 } } , 
     { "fa" , { 1 , 17 , -1 } } , 
     { "fe" , { 0 , 1 , -1 } } , 
     { "fu" , { 1 , -1 , -1 } } , 
     { "g8" , { 12 , 13 , -1 } } , 
     { "g9" , { 13 , 19 , -1 } } , 
     { "g_" , { 1 , -1 , -1 } } , 
     { "ga" , { 0 , 1 , 17 } } , 
     { "ge" , { 0 , 1 , 19 } } , 
     { "gi" , { 1 , 19 , -1 } } , 
     { "go" , { 1 , 19 , -1 } } , 
     { "gu" , { 1 , 19 , -1 } } , 
     { "gw" , { 19 , -1 , -1 } } , 
     { "h8" , { 14 , -1 , -1 } } , 
     { "h9" , { 13 , -1 , -1 } } , 
     { "he" , { 1 , -1 , -1 } } , 
     { "i_" , { 2 , -1 , -1 } } , 
     { "ie" , { 2 , -1 , -1 } } , 
     { "j8" , { 13 , -1 , -1 } } , 
     { "ja" , { 1 , 17 , 19 } } , 
     { "ji" , { 1 , 19 , -1 } } , 
     { "jo" , { 1 , -1 , -1 } } , 
     { "ju" , { 1 , -1 , -1 } } , 
     { "jy" , { 19 , -1 , -1 }} , 
     { "k_" , { 1 , -1 , -1 } } , 
     { "ke" , { 0 , -1 , -1 } } , 
     { "l8" , { 12 , 13 , -1 } } , 
     { "l9" , { 13 , -1 , -1 } } , 
     { "l_" , { 19 , -1 , -1 } } , 
     { "la" , { 0 , 17 , 19 } } , 
     { "le" , { 0 , 19 , -1 } } , 
     { "li" , { 19 , -1 , -1 } } , 
     { "lo" , { 19 , -1 , -1 } } ,
     { "ly" , { 19 , -1 , -1 } } , 
     { "m8" , { 12 , 13 , 19 } } , 
     { "m9" , { 13 , -1 , -1 } } , 
     { "ma" , { 1 , 17 , 19 } } , 
     { "me" , { 1 , -1 , -1 } } , 
     { "mi" , { 1 , -1 , -1 } } , 
     { "mo" , { 1 , -1 , -1 } } , 
     { "mu" , { 1 , -1 , -1 } } , 
     { "my" , { 19 , -1 , -1 } } , 
     { "n8" , { 13 , -1 , -1 } } , 
     { "n9" , { 19 , -1 , -1 } } , 
     { "n_" , { 1 , 19 , -1 } } , 
     { "na" , { 1 , 17 , 19 } } , 
     { "ne" , { 0 , 1 , 17 } } , 
     { "ni" , { 1 , 19 , -1 } } , 
     { "no" , { 1 , 19 , -1 } } , 
     { "ny" , { 19 , -1 , -1 } } , 
     { "oa" , { 0 , -1 , -1 } } , 
     { "ou" , { 4 , -1 , -1 } } , 
     { "ow" , { 4 , -1 , -1 } } , 
     { "p8" , { 13 , -1 , -1 } } , 
     { "pa" , { 0 , 1 , 17 } } , 
     { "pe" , { 0 , 7 , -1 } } , 
     { "pu" , { 1 , -1 , -1 } } , 
     { "q8" , { 13 , -1 , -1 } } , 
     { "q_" , { 1 , -1 , -1 } } , 
     { "qa" , { 1 , 17 , -1 } } , 
     { "qe" , { 0 , -1 , -1 } } , 
     { "qo" , { 1 , -1 , -1 } } , 
     { "r8" , { 13 , -1 , -1 } } , 
     { "ra" , { 0 , 1 , -1 } } , 
     { "re" , { 0 , -1 , -1 } } , 
     { "s8" , { 13 , -1 , -1 } } , 
     { "s9" , { 13 , 19 , -1 } } , 
     { "s_" , { 1 , 19 , -1 } } , 
     { "sa" , { 1 , 17 , 19 } } , 
     { "se" , { 1 , 17 , -1 } } , 
     { "si" , { 19 , -1 , -1 } } , 
     { "so" , { 1 , 19 , -1 } } , 
     { "sw" , { 19 , -1 , -1 } } , 
     { "sy" , { 19 , -1 , -1 } } , 
     { "t8" , { 13 , -1 , -1 } } , 
     { "t_" , { 1 , -1 , -1 } } , 
     { "ta" , { 17 , -1 , -1 } } , 
     { "te" , { 0 , 1 , -1 } } , 
     { "tu" , { 1 , -1 , -1 } } , 
     { "v8" , { 13 , -1 , -1 } } , 
     { "v_" , { 1 , -1 , -1 } } , 
     { "va" , { 17 , -1 , -1 } } , 
     { "ve" , { 0 , 1 , 17 } } , 
     { "wa" , { 16 , -1 , -1 } } , 
     { "we" , { 16 , -1 , -1 } } , 
     { "y8" , { 12 , -1 , -1 } } , 
     { "y9" , { 15 , -1 , -1 } } , 
     { "ya" , { 19 , -1 , -1 } } , 
     { "ye" , { 1 , 15 , -1 } } , 
     { "z8" , { 13 , -1 , -1 } } , 
     { "za" , { 17 , -1 , -1 } }
  } ; 

IRR irr_tbl3[_IRR3_] = {
     { "8ye" , { 15 , -1 , -1 } } , 
     { "9ye" , { 15 , -1 , -1 } } , 
     { "Lla" , { 5 , -1 , -1 } } , 
     { "Lle" , { 5 , -1 , -1 } } , 
     { "_ye" , { 15 , -1 , -1 } } , 
     { "aye" , { 15 , -1 , -1 } } , 
     { "bu_" , { 2 , -1 , -1 } } , 
     { "bue" , { 2 , -1 , -1 } } , 
     { "bw8" , { 18 , -1 , -1 } } , 
     { "bwa" , { 16 , -1 , -1 } } , 
     { "bye" , { 15 , -1 , -1 } } , 
     { "cuu" , { 4 , -1 , -1 } } , 
     { "cuw" , { 4 , -1 , -1 } } , 
     { "cwe" , { 16 , -1 , -1 } } , 
     { "cye" , { 15 , -1 , -1 } } , 
     { "daB" , { 11 , -1 , -1 } } , 
     { "daL" , { 11 , -1 , -1 } } , 
     { "daM" , { 11 , -1 , -1 } } , 
     { "daN" , { 11 , -1 , -1 } } , 
     { "dau" , { 4 , -1 , -1 } } , 
     { "daw" , { 4 , -1 , -1 } } , 
     { "deu" , { 4 , -1 , -1 } } , 
     { "dew" , { 4 , -1 , -1 } } , 
     { "dou" , { 4 , -1 , -1 } } , 
     { "dow" , { 4 , -1 , -1 } } , 
     { "duu" , { 4 , -1 , -1 } } , 
     { "duw" , { 4 , -1 , -1 } } , 
     { "dw8" , { 18 , -1 , -1 } } , 
     { "dwe" , { 16 , -1 , -1 } } , 
     { "dye" , { 15 , -1 , -1 } } , 
     { "eye" , { 15 , -1 , -1 } } , 
     { "fau" , { 4 , -1 , -1 } } , 
     { "faw" , { 4 , -1 , -1 } } , 
     { "feB" , { 11 , -1 , -1 } } , 
     { "feL" , { 11 , -1 , -1 } } , 
     { "feM" , { 11 , -1 , -1 } } , 
     { "feN" , { 11 , -1 , -1 } } , 
     { "feu" , { 4 , -1 , -1 } } , 
     { "few" , { 4 , -1 , -1 } } , 
     { "fw8" , { 18 , -1 , -1 } } , 
     { "fye" , { 15 , -1 , -1 } } , 
     { "g__" , { 2 , -1 , -1 } } , 
     { "g_e" , { 2 , -1 , -1 } } , 
     { "gaB" , { 11 , -1 , -1 } } , 
     { "gaL" , { 11 , -1 , -1 } } , 
     { "gaM" , { 11 , -1 , -1 } } , 
     { "gaN" , { 11 , -1 , -1 } } , 
     { "gau" , { 4 , -1 , -1 } } , 
     { "gaw" , { 4 , -1 , -1 } } , 
     { "geB" , { 11 , -1 , -1 } } , 
     { "geL" , { 11 , -1 , -1 } } , 
     { "geM" , { 11 , -1 , -1 } } , 
     { "geN" , { 11 , -1 , -1 } } , 
     { "geu" , { 4 , -1 , -1 } } , 
     { "gew" , { 4 , -1 , -1 } } , 
     { "giu" , { 4 , -1 , -1 } } , 
     { "giw" , { 4 , -1 , -1 } } , 
     { "gou" , { 4 , -1 , -1 } } , 
     { "gow" , { 4 , -1 , -1 } } , 
     { "guu" , { 4 , -1 , -1 } } , 
     { "guw" , { 4 , -1 , -1 } } , 
     { "gw8" , { 1 , 18 , -1 } } , 
     { "gwa" , { 1 , 16 , -1 } } , 
     { "gwe" , { 16 , -1 , -1 } } , 
     { "gye" , { 15 , -1 , -1 } } , 
     { "heu" , { 4 , -1 , -1 } } , 
     { "hew" , { 4 , -1 , -1 } } , 
     { "hwa" , { 16 , -1 , -1 } } , 
     { "hye" , { 15 , -1 , -1 } } , 
     { "iye" , { 15 , -1 , -1 } } , 
     { "ja_" , { 2 , -1 , -1 } } , 
     { "jaa" , { 2 , -1 , -1 } } , 
     { "jau" , { 4 , -1 , -1 } } , 
     { "jaw" , { 4 , -1 , -1 } } , 
     { "je_" , { 2 , -1 , -1 } } , 
     { "jee" , { 2 , -1 , -1 } } , 
     { "jeu" , { 4 , -1 , -1 } } , 
     { "jew" , { 4 , -1 , -1 } } , 
     { "ji_" , { 2 , -1 , -1 } } , 
     { "jie" , { 2 , -1 , -1 } } , 
     { "juu" , { 4 , -1 , -1 } } , 
     { "juw" , { 4 , -1 , -1 } } , 
     { "jw8" , { 18 , -1 , -1 } } , 
     { "jwa" , { 16 , -1 , -1 } } , 
     { "jwe" , { 16 , -1 , -1 } } , 
     { "jye" , { 15 , -1 , -1 } } , 
     { "kwe" , { 16 , -1 , -1 } } , 
     { "kye" , { 15 , 17 , -1 } } , 
     { "laB" , { 11 , -1 , -1 } } , 
     { "laL" , { 11 , -1 , -1 } } , 
     { "laM" , { 11 , -1 , -1 } } , 
     { "laN" , { 11 , -1 , -1 } } , 
     { "lau" , { 4 , -1 , -1 } } , 
     { "law" , { 4 , -1 , -1 } } , 
     { "leB" , { 11 , -1 , -1 } } , 
     { "leL" , { 11 , -1 , -1 } } , 
     { "leM" , { 11 , -1 , -1 } } , 
     { "leN" , { 11 , -1 , -1 } } , 
     { "leu" , { 4 , -1 , -1 } } , 
     { "lew" , { 4 , -1 , -1 } } , 
     { "liu" , { 4 , -1 , -1 } } , 
     { "liw" , { 4 , -1 , -1 } } , 
     { "lou" , { 4 , -1 , -1 } } , 
     { "low" , { 4 , -1 , -1 } } , 
     { "lw8" , { 18 , -1 , -1 } } , 
     { "lwe" , { 16 , -1 , -1 } } , 
     { "lye" , { 15 , -1 , -1 } } , 
     { "maB" , { 11 , -1 , -1 } } , 
     { "maL" , { 11 , -1 , -1 } } , 
     { "maM" , { 11 , -1 , -1 } } , 
     { "maN" , { 11 , -1 , -1 } } , 
     { "mau" , { 4 , -1 , -1 } } , 
     { "maw" , { 4 , -1 , -1 } } , 
     { "meB" , { 11 , -1 , -1 } } , 
     { "meL" , { 11 , -1 , -1 } } , 
     { "meM" , { 11 , -1 , -1 } } , 
     { "meN" , { 11 , -1 , -1 } } , 
     { "miu" , { 4 , -1 , -1 } } , 
     { "miw" , { 4 , -1 , -1 } } , 
     { "mu_" , { 2 , -1 , -1 } } , 
     { "mue" , { 2 , -1 , -1 } } , 
     { "mwa" , { 16 , -1 , -1 } } , 
     { "mye" , { 1 , 15 , -1 } } , 
     { "n8u" , { 4 , -1 , -1 } } , 
     { "n8w" , { 4 , -1 , -1 } } , 
     { "na_" , { 2 , -1 , -1 } } , 
     { "naa" , { 2 , -1 , -1 } } , 
     { "nau" , { 4 , -1 , -1 } } , 
     { "naw" , { 4 , -1 , -1 } } , 
     { "nuu" , { 4 , -1 , -1 } } , 
     { "nuw" , { 4 , -1 , -1 } } , 
     { "nwe" , { 16 , -1 , -1 } } , 
     { "nye" , { 15 , -1 , -1 } } , 
     { "oye" , { 15 , -1 , -1 } } , 
     { "pwe" , { 16 , -1 , -1 } } , 
     { "pye" , { 15 , -1 , -1 } } , 
     { "q__" , { 2 , -1 , -1 } } , 
     { "q_e" , { 2 , -1 , -1 } } , 
     { "qau" , { 4 , -1 , -1 } } , 
     { "qaw" , { 4 , -1 , -1 } } , 
     { "qeu" , { 4 , -1 , -1 } } , 
     { "qew" , { 4 , -1 , -1 } } , 
     { "qwa" , { 16 , -1 , -1 } } , 
     { "qwe" , { 16 , -1 , -1 } } , 
     { "qye" , { 15 , -1 , -1 } } , 
     { "rye" , { 15 , -1 , -1 } } , 
     { "s_u" , { 4 , -1 , -1 } } , 
     { "s_w" , { 4 , -1 , -1 } } , 
     { "seu" , { 4 , -1 , -1 } } , 
     { "sew" , { 4 , -1 , -1 } } , 
     { "sw8" , { 18 , -1 , -1 } } , 
     { "swe" , { 16 , -1 , -1 } } , 
     { "sye" , { 15 , -1 , -1 } } , 
     { "tau" , { 4 , -1 , -1 } } , 
     { "taw" , { 4 , -1 , -1 } } , 
     { "teu" , { 4 , -1 , -1 } } , 
     { "tew" , { 4 , -1 , -1 } } , 
     { "twe" , { 16 , -1 , -1 } } , 
     { "tye" , { 15 , -1 , -1 } } , 
     { "uye" , { 15 , -1 , -1 } } , 
     { "vuu" , { 4 , -1 , -1 } } , 
     { "vuw" , { 4 , -1 , -1 } } , 
     { "vw8" , { 18 , -1 , -1 } } , 
     { "vwa" , { 16 , -1 , -1 } } , 
     { "vwe" , { 16 , -1 , -1 } } , 
     { "yaB" , { 11 , -1 , -1 } } , 
     { "yaL" , { 11 , -1 , -1 } } , 
     { "yaM" , { 11 , -1 , -1 } } , 
     { "yaN" , { 11 , -1 , -1 } } , 
     { "yeB" , { 11 , -1 , -1 } } , 
     { "yeL" , { 11 , -1 , -1 } } , 
     { "yeM" , { 11 , -1 , -1 } } , 
     { "yeN" , { 11 , -1 , -1 } } , 
     { "yeu" , { 4 , -1 , -1 } } , 
     { "yew" , { 4 , -1 , -1 } } , 
     { "zau" , { 4 , -1 , -1 } } , 
     { "zaw" , { 4 , -1 , -1 } } , 
     { "zeu" , { 4 , -1 , -1 } } , 
     { "zew" , { 4 , -1 , -1 } } , 
     { "zou" , { 4 , -1 , -1 } } , 
     { "zow" , { 4 , -1 , -1 } } , 
     { "zuu" , { 4 , -1 , -1 } } , 
     { "zuw" , { 4 , -1 , -1 } } , 
     { "zw8" , { 18 , -1 , -1 } } , 
     { "zwa" , { 16 , -1 , -1 } } , 
     { "zwe" , { 16 , -1 , -1 } } , 
     { "zye" , { 15 , -1 , -1 } }
  } ; 

IRR irr_tbl4[_IRR4_] = {
     { "buL_" , { 3 , -1 , -1 } } , 
     { "buLe" , { 3 , -1 , -1 } } , 
     { "byeu" , { 4 , -1 , -1 } } , 
     { "byew" , { 4 , -1 , -1 } } , 
     { "d_L_" , { 3 , -1 , -1 } } , 
     { "d_Le" , { 3 , -1 , -1 } } , 
     { "daL_" , { 3 , -1 , -1 } } , 
     { "daLa" , { 3 , -1 , -1 } } , 
     { "geL_" , { 3 , -1 , -1 } } , 
     { "geLe" , { 3 , -1 , -1 } } , 
     { "giL_" , { 3 , -1 , -1 } } , 
     { "giLe" , { 3 , -1 , -1 } } , 
     { "gyeu" , { 4 , -1 , -1 } } , 
     { "gyew" , { 4 , -1 , -1 } } , 
     { "haye" , { 8 , -1 , -1 } } , 
     { "keL_" , { 3 , -1 , -1 } } , 
     { "keLe" , { 3 , -1 , -1 } } , 
     { "lyeu" , { 4 , -1 , -1 } } , 
     { "lyew" , { 4 , -1 , -1 } } , 
     { "muL_" , { 3 , -1 , -1 } } , 
     { "muLe" , { 3 , -1 , -1 } } , 
     { "nuL_" , { 3 , -1 , -1 } } , 
     { "nuLe" , { 3 , -1 , -1 } } , 
     { "seLu" , { 4 , -1 , -1 } } , 
     { "seLw" , { 4 , -1 , -1 } } , 
     { "siL_" , { 3 , -1 , -1 } } , 
     { "siLe" , { 3 , -1 , -1 } } , 
     { "swuu" , { 4 , -1 , -1 } } , 
     { "swuw" , { 4 , -1 , -1 } } 
  } ; 
  
IRR irr_tbl5[_IRR5_] = {
     { "bogel" , { 9 , -1 , -1 } } ,
     { "gagel" , { 9 , -1 , -1 } } , 
     { "gyeL_" , { 3 , -1 , -1 } } , 
     { "gyeLe" , { 3 , -1 , -1 } } , 
     { "iVgel" , { 9 , -1 , -1 } } , 
     { "il_le" , { 6 , -1 , -1 } } , 
     { "jagel" , { 9 , -1 , -1 } } , 
     { "nagel" , { 9 , -1 , -1 } } , 
     { "onela" , { 10 , -1 , -1 } } , 
     { "ul_le" , { 6 , -1 , -1 } }
  } ; 

IRRTABLE irrTable[5] = {
  { irr_tbl1 , _IRR1_ } , 
  { irr_tbl2 , _IRR2_ } , 
  { irr_tbl3 , _IRR3_ } , 
  { irr_tbl4 , _IRR4_ } , 
  { irr_tbl5 , _IRR5_ } 
 } ;


