/* dictionary opeartions in MAK
 *
 * create : 95.5.27 (Sat), bgjang@csone
 * modify :
 * 95.5.28 (Sun), bgjang@csone - can handle normal type dictionary well.
 * 95.5.29 (Mon), bgjang@csone - functions for linear access to words in dict.
 * 95.5.31 (Wed), bgjang@csone - add freq member to cellTypeN structure
 * 95.6.2 (Fri), bgjang@csone - MAdictNKey*() functions added for SI, SO
 *                              in keyword.
 * 95.6.6 (Tue), bgjang@csone - E-type dictionary routines
 * 95.6.7 (Wed), bgjang@csone - E-type dictionary routines complete
 *                              without delete command
 * 95.6.13 (Tue), bgjang@csone - add EPOSstr
 *
 */
#include <stdio.h>
#include "../pos.h"
#include "../irrtbl.h"
#include "../common.h"
#include "../MAerror.h"
#include <dictAPI.h>
#include <MAdict.h>
#include <MAerror.h>


/* global definitions ------------------------------------------------*/

/* dictionary type currently supported
 * and declared in MAdict.h
 */
char *dictTypeStr[] = { "dictTypeNone", "dictTypeNormal",
                        "dictTypeExce", "dictTypeIrregular" };


/* POS string definitions
 * and the declaration of POSstr in pos.h
 */
char *POSstr[] = {
     "INI", 
     "s,", "s.", "s`", "s'", "s-", "su", "sw", "sy", 
     "f", 
     "nnn", "nct", "nca", "ncs", "nc", "nq", "nbu", "nb", "npp", "npd", "nn", 
     "pv", "pad", "pa", "px", 
     "md", "mn", "m", 
     "at", "ad", "ajw", "ajs", "a", 
     "i", 
     "jc", "jcm", "jcv", "jca", "jcp", "jx", "jj", 
     "ecq", "ecs", "ecx", "exm", "exn", "exa", "efp", "ef", 
     "xn", "xpv", "xpa", "xa",
     "FIN", 
     "INT", "usy", "uf", "unnn",
     NULL };

/* EPOS string definitions
 * and the declaration of POSstr in pos.h
 */
char *EPOSstr[] = {
     "INI", 
     "s,", "s.", "s`", "s'", "s-", "su", "sw", "sy", 
     "f", 
     "nnn", "nct", "nca", "ncs", "nc", "nq", "nbu", "nb", "npp", "npd", "nn", 
     "pv", "pad", "pa", "px", 
     "md", "mn", "m", 
     "at", "ad", "ajw", "ajs", "a", 
     "i", 
     "jc", "jcm", "jcv", "jca", "jcp", "jx", "jj", 
     "ecq", "ecs", "ecx", "exm", "exn", "exa", "efp", "ef", 
     "xn", "xpv", "xpa", "xa",
     "FIN", 
     "INT", "usy", "uf", "unnn",
     "?P", "?B", "??", "NONE",
     NULL };


/* local definitions -----------------------------------------------*/

/* cell type for dictTypeNormal
 *
 * Note : if cell type for normal dictionary is changed then
 *        find CHANGE_CELLN, and change a few code in that position
 */
typedef struct {
    enum POS pos;
    float freq;
} cellTypeN;

/* cell type for dictTypeExec
 * 
 * An interpreted data (long form for shorted word) is
 * composed of (word,POS) pairs. And, one word can have
 * more than two interpreted datum.
 *
 * The next type represent one (word,POS) pair. And
 * nextData points the starting index of the next
 * interpreted data. Otherwise, it's value is -1.
 */
typedef struct {
    int nextData;
    char kimmoStr[MAX_WORD_LEN];
    enum EPOS epos;
} cellTypeE;


typedef struct {
    enum POS pos;
} cellTypeU;

typedef struct {
    int irr;        /* 4-byte bit vector */
} cellTypeI;



/* static variables -------------------------------------------------*/

/* id2type array for type checking for each dictionary
 */
static enum dictType id2type[MAX_OPEN_DICT];



/* dictionary interfaces --------------------------------------------*/

void MAdictInit()
{
    register int i;

    dictInit();

    for( i=0; i<MAX_OPEN_DICT; i++ )
        id2type[i] = dictTypeNone;
}

// return dict ID if success

int MAdictOpen( path, flag, mode, type )
    char *path;
    int flag;
    int mode;
    enum dictType type;   /* dictionary type */
{
    int r;

    // for error handling
    setFuncName( "MAdictOpen" );

    // call dict API's open opeartion
    r = dictOpen( path, flag, mode );

    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    id2type[r] = type;
    return r;
}

void MAdictClose( id )
    int id;
{
    int r;

    r = dictClose( id );
    id2type[id] = dictTypeNone;

    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );
}


/* Normal dictionary operations -------------------------------*/

// CHANGE_CELLN : if cellTypeN is changed, then this function's
//                argument must be changed, I think.
//
int MAdictNInsertPOS( id, word, pos, freq )
    int id;
    char *word;
    enum POS pos;
    float freq;
{
    register int i;
    int r;

    checkDictType( id, dictTypeNormal );

    setFuncName( "MAdictNInsertPOS" );

    // call dict API's find operation
    r = dictFind( id, word );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) {

        cellTypeN *t;

        debugMesg("MAdictNInsertPOS, firstly insert POS with keyword %s.\n", word);
        // allocate temporary variable
        t = (cellTypeN*) malloc(sizeof(cellTypeN));
        if( t == NULL )
            MAerror( END, E_MALLOC, __FILE__, __LINE__, "malloc() failed.\n" );

        t[0].pos = pos;
        t[0].freq = freq;        // CHANGE_CELLN

        // set global var.
        r = dictResultSet( t, sizeof(cellTypeN) );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        // free temporarily used malloced variable
        free( t );

        // dictionary insert
        r = dictInsert( id, word );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
    else if(r == D_ERR_NO ) {   // data is existed and in global var.

        cellTypeN *t, *g; int posNum;

        debugMesg("data is existed, so append this POS at the end of contents.\n");
        posNum = dictResultLen / sizeof(cellTypeN);
        g = (cellTypeN*) dictResult;

        // check this POS is existed in contents or not
        for( i=0; i<posNum; i++ )
            if( g[i].pos == pos ) {
                MAerror( NOT, E_EXIST_POS, __FILE__, __LINE__,
                  "word %s have already POS %s in it.\n", word, POSstr[pos] );
                return FALSE;
            }

        // allocate temp var.
        t = (cellTypeN*) malloc( (posNum+1) * sizeof(cellTypeN) );
        if( t == NULL )
            MAerror( END, E_MALLOC, __FILE__, __LINE__, "malloc() failed.\n" );

        // copy the previous one to temp var.
        for( i=0; i<posNum; i++ )
            memcpy( &(t[i]), &(g[i]), sizeof(cellTypeN) );
            // t[i].pos = g[i].pos;   // memcpy() can be used alternatively

        // add new one to temp var.
        t[posNum].pos = pos;
        t[posNum].freq = freq;     // CHANGE_CELLN

        debugMesg("Now, %s pos %f freq is inserted and number of pos is %d.\n", POSstr[pos], freq, posNum+1);
        // set global var.
        r = dictResultSet( t, (posNum+1) * sizeof(cellTypeN) );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        // free temporarily used malloced variable
        free( t );

        // modify current list of POS in dictionary
        r = dictModify( id, word );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

int MAdictNDeletePOS( id, word, pos )
    int id;
    char *word;
    enum POS pos;
{
    register int i,j;
    int r;

    checkDictType( id, dictTypeNormal );

    setFuncName( "MAdictNDeletePOS" );

    // find one
    r = dictFind( id, word );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) {
        MAerror( NOT, E_DICT + r, __FILE__, __LINE__,
          "no proper word is existed in database.\n" );
        return FALSE;
    }
    else if( r == D_ERR_NO ) {
        cellTypeN *t, *g; int posNum;

        posNum = dictResultLen / sizeof(cellTypeN);
        g = (cellTypeN*) dictResult;

        // check whether this POS is existed or not
        for( i=0; i<posNum; i++ )
            if( g[i].pos == pos ) break;

        // if not existed
        if( i == posNum ) {
            MAerror( NOT, E_NO_POS, __FILE__, __LINE__,
              "no proper POS is in the contents of word.\n" );
            return FALSE;
        }

        // allocated temp var.
        t = (cellTypeN*) malloc ( (posNum-1)*sizeof(cellTypeN) );
        if( t == NULL )
            MAerror( END, E_MALLOC, __FILE__, __LINE__, "malloc() failed.\n" );

        // copy POS properly from global var. to temp var.
        for( i=0, j=0; i<posNum; i++ )
            if( g[i].pos != pos )
                memcpy( &(t[j++]), &(g[i].pos), sizeof(cellTypeN) );
                // t[j++].pos = g[i].pos;

        // set global var.
        r = dictResultSet( t, (posNum-1) * sizeof(cellTypeN) );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        // free temporarily used malloced variable
        free( t );

        // modify current list of POS in dictionary
        r = dictModify( id, word );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

int MAdictNZapWord( id, word )
    int id;
    char *word;
{
    int r;

    checkDictType( id, dictTypeNormal );

    setFuncName( "MAdictNZapWord" );

    // delete 
    r = dictDelete( id, word );
    if( r < 0 ) {
        MAerror( NOT, E_DICT + r, __FILE__, __LINE__,
          "Word is not existed or delete operation is failed.\n");
        return FALSE;
    }
    return TRUE;
}

/* this routine is used for cache for normal dictionary
 * routines
 *
 * if it cannot find, then don't touch the values in freqArray
 * and return FALSE.
 */
int MAdictNGetPOSArray( id, word, freqArray )
    int id;
    char *word;
    float freqArray[];
{
    register int i;
    int r;
    int c;

    checkDictType( id, dictTypeNormal );

    setFuncName( "MAdictNGetPOSArray" );

    // find contents
    r = dictFind( id, word );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    // found
    if( r == D_ERR_NO ) {
        cellTypeN *g; int posNum;

        posNum = dictResultLen / sizeof(cellTypeN);
        g = (cellTypeN*) dictResult;

        for( i=0; i<NUM_POS; i++ ) freqArray[i] = 0;

        for( i=0; i<posNum; i++ )
            freqArray[ g[i].pos ] = (float) g[i].freq;

        return TRUE;
    }

    // not found
    else if( r == D_NO_DATA ) {
        /* If there is no other POS and it can be interpreted as
         * sy, nnn, f, then it must be returned as sy/nnn/f
         */
        for( i=0, c=0; i<strlen(word); i++ )
            if( word[i] == CTRL(N) || word[i] == CTRL(O) ) c++;

        if( c == 2 && word[0] == CTRL(N) && word[strlen(word)-1] == CTRL(O) ) {
            if( isalpha(word[1]) ) freqArray[uf] = DEFAULT_FREQ;
            else if( strlen(word) == 3 && !isdigit(word[1]) ) freqArray[usy] = DEFAULT_FREQ;
            else freqArray[unnn] = DEFAULT_FREQ;

            return TRUE;
        }
        else
            return FALSE; // return E_NO_WORD;
    }
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}


int MAdictNGetIthPOS( id, word, ith, pType, pFreq )
    int id;
    char *word;
    int ith;          // IN : 0 .. posNum-1
    enum POS *pType;  // OUT
    float *pFreq;       // OUT
{
    register int i;
    int r;
    int c;

    checkDictType( id, dictTypeNormal );

    setFuncName( "MAdictNInsertPOS" );

    // find contents
    r = dictFind( id, word );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    // found
    if( r == D_ERR_NO ) {
        cellTypeN *g; int posNum;

        posNum = dictResultLen / sizeof(cellTypeN);

        // check ith value is proper.
        if( posNum-1 < ith ) return FALSE; // return E_BIG_ITH;

        // set output argument
        g = (cellTypeN*) dictResult;
        *pType = g[ith].pos;
        *pFreq = g[ith].freq;
        return TRUE;
    }

    // not found
    else if( r == D_NO_DATA ) {
        /* If there is no other POS and it can be interpreted as
         * sy, nnn, f, then it must be returned as sy/nnn/f
         */
        if( ith == 0 ) {
            for( i=0, c=0; i<strlen(word); i++ )
                if( word[i] == CTRL(N) || word[i] == CTRL(O) ) c++;

            if( c == 2 && word[0] == CTRL(N) && word[strlen(word)-1] == CTRL(O) ) {
                if( isalpha(word[1]) ) *pType = uf;
                else if( strlen(word) == 3 && !isdigit(word[1]) ) *pType = usy;
                else *pType = unnn;
                *pFreq = DEFAULT_FREQ;

                return TRUE;
            }
        }
        else
            return FALSE; // return E_NO_WORD;
    }
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

// right ? hmm.
// return value "װ exm 12 run 4"
//
int MAdictNGetFirstWord( id, s )
    int id;
    char *s; // OUT
{
    register int i, j;
    char keyword[128], keywordKS[128];
    cellTypeN *g;
    int posNum;
    int r;

    checkDictType( id, dictTypeNormal );

    setFuncName( "MAdictNGetFirstWord" );

    r = dictGetFirstWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    // data is existed without an error
    // delete SI(^N), SO(^O)
    MAdictNKey2KS( keyword, keywordKS );
    sprintf( s, "%s", keywordKS );

    // print POS following the keyword
    posNum = dictResultLen / sizeof(cellTypeN);
    g = (cellTypeN*) dictResult;

    for( i=0; i<posNum; i++ ) {     // CHANGE_CELLN
        char freqStr[16];

        strcat( s, " " );
        strcat( s, POSstr[g[i].pos] );
        sprintf( freqStr, " %f", g[i].freq );
        strcat( s, freqStr );
    }
    return TRUE;
}

int MAdictNGetNextWord( id, s )
    int id;
    char *s;   // OUT
{
    register int i,j;
    char keyword[128], stripKey[128], keywordKS[128];
    cellTypeN *g;
    int posNum;
    int r;

    checkDictType( id, dictTypeNormal );

    setFuncName( "MAdictNGetNextWord" );

    r = dictGetNextWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    // data is existed without an error
    // delete SI(^N), SO(^O)
    MAdictNKey2KS( keyword, keywordKS );
    sprintf( s, "%s", keywordKS );

    // print POS following the keyword
    posNum = dictResultLen / sizeof(cellTypeN);
    g = (cellTypeN*) dictResult;

    for( i=0; i<posNum; i++ ) {      // CHANGE_CELLN
        char freqStr[16];

        strcat( s, " " );
        strcat( s, POSstr[g[i].pos] );
        sprintf( freqStr, " %f", g[i].freq );
        strcat( s, freqStr );
    }
    return TRUE;
}

void MAdictNKey2KS( s, ks )
    char *s;       // IN : including SI, SO and in kimmo code
    char *ks;      // OUT : delete SI, SO and converting to KS code
{
    register int i, j;
    char t[256], u[256];

    strcpy( ks, "" );

    for( i=0, j=0; i<strlen(s); i++ ) {
        if( s[i] == CTRL(N) ) {   // case Korean : T_KOR
            if( j != 0 ) {
                t[j] = '\0';   j = 0;
                kimmo2ks( t, u );
                strcat( ks, u );
            }
            continue;
        }
        if( s[i] == CTRL(O) ) {  // the other case : T_ENG, T_SYM, T_NUM
            if( j != 0 ) {
                t[j] = '\0';   j = 0;
                strcat( ks, t );
            }
            continue;
        }
        t[j++] = s[i];
    }
    if( j != 0 ) {
        t[j] = '\0';   j = 0;
        kimmo2ks( t, u );
        strcat( ks, u );
    }
}

void MAdictNKey2Kimmo( s, kimmo )
    char *s;      // IN : including SI, SO and in kimmo code
    char *kimmo;  // OUT : delete SI, SO only
{
    register int i, j;

    for( i=0, j=0; i<strlen(s); i++ )
        if( s[i] != CTRL(N) && s[i] != CTRL(O) )
            kimmo[j++] = s[i];
    kimmo[j] = '\0';
}

/* Exceptional dictionary operations --------------------------*/

int _MAdictEEqualData( p, q, numOfElts )
    cellTypeE *p, *q;
    int numOfElts;
{
    register int i;

    for( i=0; i<numOfElts; i++ ) {
        if( strcmp(p[i].kimmoStr, q[i].kimmoStr) ) return FALSE;
        if( p[i].epos != q[i].epos ) return FALSE;
    }
    debugMesg("_MAdictEEqualData : TRUE is returned.\n");
    return TRUE;
}

MAdictEInsertData( id, word, leftMark, rightMark, data )
    int id;
    char *word;     // in kimmo code
    char leftMark;  // $, +
    char rightMark; // $, +
    char *data;     // available delimiter : " /+\n"
{
    register int i;
    int r;
    int lastNextData;
    char keyword[128];
    char tData[512], *pData;

    checkDictType( id, dictTypeExce );
    setFuncName( "MAdictEInsertData" );
    
    // keyword form
    // "leftMark word rightMark". ex) "$ hagi $"
    sprintf(keyword, "%c %s %c", leftMark, word, rightMark);

    r = dictFind( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    // copy data to strtok()
    strcpy( tData, data );
    pData = tData;

    // firstly inserted data
    if( r == D_NO_DATA ) {
        cellTypeE t[MAX_CELL_FOR_INTERPRETED_DATA];

        debugMesg("MAdictEInsertData, firstly insert Data with keyword %s.\n", keyword );

        for( i=0; 1 ; i++ ) {
            char *p, *q; int tEpos;

            t[i].nextData = -1;
            p = (char*) strtok( pData, " +/\n" ); pData = NULL; if( !p ) break;
            q = (char*) strtok( pData, " +/\n" );
            if( !q )
                MAerror( END, E_ETYPE_DATA, __FILE__, __LINE__,
                  "input data's format is wrong.\n" );

            tEpos = str2epos( q );
            if( tEpos == -1 )
                MAerror( END, E_EPOS, __FILE__, __LINE__,
                  "input data include wrong EPOS value.\n" );

            strcpy( t[i].kimmoStr, p );
            t[i].epos = (enum EPOS) tEpos;
        }

        r = dictResultSet( t, sizeof(cellTypeE) * i );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        r = dictInsert( id, keyword );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }

    // data existed
    else if( r == D_ERR_NO ) {
        cellTypeE t[MAX_CELL_FOR_INTERPRETED_DATA * MAX_IDATA_FOR_KEY];
        int numOfCellOld, numOfCellNew, numOfCellDiff;

        debugMesg("MAdictEInsertData, data is existed, appened at the end.\n");

        // copy previous result
        memcpy( t, dictResult, dictResultLen );

        numOfCellOld = dictResultLen/sizeof(cellTypeE);

        for( i=numOfCellOld; 1 ; i++ ) {
            char *p, *q; int tEpos;

            t[i].nextData = -1;
            p = (char*) strtok( pData, " +/\n" ); pData = NULL; if( !p ) break;
            q = (char*) strtok( pData, " +/\n" );
            if( !q )
                MAerror( END, E_ETYPE_DATA, __FILE__, __LINE__,
                  "input data's format is wrong.\n" );

            tEpos = str2epos( q );
            if( tEpos == -1 )
                MAerror( END, E_EPOS, __FILE__, __LINE__,
                  "input data include wrong EPOS value.\n" );

            strcpy( t[i].kimmoStr, p );
            t[i].epos = (enum EPOS) tEpos;
        }

        numOfCellNew = i;

        numOfCellDiff = numOfCellNew - numOfCellOld;

        // check the current data is in dictionary
        for( i=0; i != -1; i = t[i].nextData) {
            int numOfElts;

            // check the size
            if( t[i].nextData == -1 ) {
                numOfElts = numOfCellOld - i;
                lastNextData = i;
            }
            else
                numOfElts = t[i].nextData - i;

            if( numOfCellDiff != numOfElts ) continue;

            // if the size is equal
            r = _MAdictEEqualData( &(t[i]), &(t[numOfCellOld]), numOfCellDiff );
            if( r == TRUE ) {
                MAerror( NOT, E_EXIST_DATA, __FILE__, __LINE__,
                  "Input data is in dictionary.\n" );
                return FALSE;
            }
        }
        t[lastNextData].nextData = numOfCellOld;

        r = dictResultSet( t, sizeof(cellTypeE) * numOfCellNew);
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        r = dictModify( id, keyword );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
}


MAdictEDeleteIthData( id, word, leftMark, rightMark, ith )
    int id;
    char *word;
    char leftMark;
    char rightMark;
    int ith;
{
}


MAdictEGetData( id, word, leftMark, rightMark, s )
    int id;
    char *word;
    char leftMark;
    char rightMark;
    char *s;
{
    register int i;
    int r;
    int nextIndex;
    char keyword[128];
    cellTypeE *p;

    checkDictType( id, dictTypeExce );
    setFuncName( "MAdictEGetData" );

    sprintf( keyword, "%c %s %c", leftMark, word, rightMark );

    r = dictFind( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    p = (cellTypeE*) dictResult;

    strcpy( s, "" );
    nextIndex = 0;
    for( i=0; i<dictResultLen/sizeof(cellTypeE); i++ ) {
        char tStr[128];
        if( i == nextIndex ) {
            if( i != 0 )
                strcat( s, "/" );
            nextIndex = p[i].nextData;
        }

        sprintf( tStr, "%s %s ", p[i].kimmoStr, EPOSstr[p[i].epos] );
        strcat( s, tStr );
    }
    return TRUE;
}

MAdictEExistData( id, word, leftMark, rightMark )
    int id;
    char* word;
    char leftMark;
    char rightMark;
{
    int r;
    char keyword[128];

    checkDictType( id, dictTypeExce );
    setFuncName( "MAdictEExistData" );

    sprintf( keyword, "%c %s %c", leftMark, word, rightMark );

    r = dictFind( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;
    return TRUE;
}

MAdictEGetIthData( id, word, leftMark, rightMark, ith, s )
    int id;
    char *word;
    char leftMark;
    char rightMark;
    int ith;
    char *s;
{
    register int ind, i;
    int r;
    char data[1024];
    int numOfElts;
    char *p;

    checkDictType( id, dictTypeExce );
    setFuncName( "MAdictEGetIthData" );
    
    r = MAdictEGetData( id, word, leftMark, rightMark, data );
    if( r == FALSE ) return FALSE;

    p = (char*) strtok( data, "/" );
    for( ;ith != 0; ith-- ) {
        p = (char*) strtok( NULL, "/" );
        if( p == NULL ) break;
    }
    if( ith == 0 ) {
        strcpy( s, p );
        return TRUE;
    }
    return FALSE;
}

int MAdictEGetFirstWord( id, s )
    int id;
    char *s; // OUT
{
    register int i, j;
    int r;
    char keyword[128]; 
    char lm, rm, word[64];
    char data[1024];

    checkDictType( id, dictTypeExce );
    setFuncName( "MAdictEGetFirstWord" );

    r = dictGetFirstWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    r = sscanf( keyword, "%c %s %c", &lm, word, &rm );
    if( r != 3 )
        MAerror( END, E_INTERNAL, __FILE__, __LINE__,
            "Internal Error : keyword of E-type dictionary have wrong format.\n" );

    r = MAdictEGetData( id, word, lm, rm, data );
    if( r == FALSE )
        MAerror( END, E_INTERNAL, __FILE__, __LINE__,
            "Internal Error : data is not existed for existed keyword.\n" );

    sprintf( s, "%c%s%c %s", lm, word, rm, data );
    return TRUE;
}

int MAdictEGetNextWord( id, s )
    int id;
    char *s; // OUT
{
    register int i, j;
    int r;
    char keyword[128]; 
    char lm, rm, word[64];
    char data[1024];

    checkDictType( id, dictTypeExce );
    setFuncName( "MAdictEGetNextWord" );

    r = dictGetNextWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    r = sscanf( keyword, "%c %s %c", &lm, word, &rm );
    if( r != 3 )
        MAerror( END, E_INTERNAL, __FILE__, __LINE__,
            "Internal Error : keyword of E-type dictionary have wrong format.\n" );

    r = MAdictEGetData( id, word, lm, rm, data );
    if( r == FALSE )
        MAerror( END, E_INTERNAL, __FILE__, __LINE__,
            "Internal Error : data is not existed for existed keyword.\n" );

    sprintf( s, "%c%s%c %s", lm, word, rm, data );
    return TRUE;
}


/* Unknown word dictionary ------------------------------------------*/

MAdictUInsertUnknown( id, word, epos, upos )
    int id;
    char *word;
    enum EPOS epos;
    enum POS upos;
{
    register int i;
    int r;
    char keyword[128];
    cellTypeU U;

    checkDictType( id, dictTypeUnknown );

    setFuncName( "MAdictUInsertUnknown" );

    sprintf( keyword, "%s %s", word, EPOSstr[epos] );

    // call dict API's find operation
    r = dictFind( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) {

        debugMesg("MAdictUInsertUnknown, firstly insert POS with keyword %s.\n", keyword);

        U.pos  =  upos;

        // set global var.
        r = dictResultSet( &U, sizeof(cellTypeU) );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        // dictionary insert
        r = dictInsert( id, keyword );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
    else if(r == D_ERR_NO ) {   // data is existed and in global var.

        cellTypeU *t, *g; int posNum;

        debugMesg("data is existed, so append this POS at the end of contents.\n");
        posNum = dictResultLen / sizeof(cellTypeU);
        g = (cellTypeU*) dictResult;

        // check this POS is existed in contents or not
        for( i=0; i<posNum; i++ )
            if( g[i].pos == upos ) {
                MAerror( NOT, E_EXIST_POS, __FILE__, __LINE__,
                  "word %s have already POS %s in it.\n", keyword, POSstr[upos] );
                return FALSE;
            }

        // allocate temp var.
        t = (cellTypeU*) malloc( (posNum+1) * sizeof(cellTypeU) );
        if( t == NULL )
            MAerror( END, E_MALLOC, __FILE__, __LINE__, "malloc() failed.\n" );

        // copy the previous one to temp var.
        for( i=0; i<posNum; i++ )
            memcpy( &(t[i]), &(g[i]), sizeof(cellTypeU) );
            // t[i].pos = g[i].pos;   // memcpy() can be used alternatively

        // add new one to temp var.
        t[posNum].pos = upos;

        debugMesg("Now, %s pos is inserted and number of pos is %d.\n", POSstr[upos], posNum+1);
        // set global var.
        r = dictResultSet( t, (posNum+1) * sizeof(cellTypeU) );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        // free temporarily used malloced variable
        free( t );

        // modify current list of POS in dictionary
        r = dictModify( id, keyword );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

MAdictUDeletePOS( id, word, epos, pos )
    int id;
    char *word;
    enum EPOS epos;
    enum POS pos;
{
    register int i,j;
    int r;
    char keyword[128];

    checkDictType( id, dictTypeUnknown );

    setFuncName( "MAdictUDeletePOS" );

    sprintf( keyword, "%s %s", word, EPOSstr[epos] );

    // find one
    r = dictFind( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) {
        MAerror( NOT, E_DICT + r, __FILE__, __LINE__,
          "no proper word is existed in database.\n" );
        return FALSE;
    }
    else if( r == D_ERR_NO ) {
        cellTypeU *t, *g; int posNum;

        posNum = dictResultLen / sizeof(cellTypeU);
        g = (cellTypeU*) dictResult;

        // check whether this POS is existed or not
        for( i=0; i<posNum; i++ )
            if( g[i].pos == pos ) break;

        // if not existed
        if( i == posNum ) {
            MAerror( NOT, E_NO_POS, __FILE__, __LINE__,
              "no proper POS is in the contents of word.\n" );
            return FALSE;
        }

        // allocated temp var.
        t = (cellTypeU*) malloc ( (posNum-1)*sizeof(cellTypeU) );
        if( t == NULL )
            MAerror( END, E_MALLOC, __FILE__, __LINE__, "malloc() failed.\n" );

        // copy POS properly from global var. to temp var.
        for( i=0, j=0; i<posNum; i++ )
            if( g[i].pos != pos )
                memcpy( &(t[j++]), &(g[i].pos), sizeof(cellTypeU) );
                // t[j++].pos = g[i].pos;

        // set global var.
        r = dictResultSet( t, (posNum-1) * sizeof(cellTypeU) );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        // free temporarily used malloced variable
        free( t );

        // modify current list of POS in dictionary
        r = dictModify( id, keyword );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

MAdictUZap( id, word, epos )
    int id;
    char *word;
    enum EPOS epos;
{
    int r;
    char keyword[128];

    checkDictType( id, dictTypeUnknown );

    setFuncName( "MAdictUZap" );

    sprintf( keyword, "%s %s", word, EPOSstr[epos] );

    // delete 
    r = dictDelete( id, keyword );
    if( r < 0 ) {
        MAerror( NOT, E_DICT + r, __FILE__, __LINE__,
          "Word is not existed or delete operation is failed.\n");
        return FALSE;
    }
    return TRUE;
}

int MAdictUGetIthPOS( id, word, epos, ith, pPos )
    int id;
    char *word;
    enum EPOS epos;
    int ith;
    enum POS * pPos;
{
    register int i;
    int r;
    char keyword[128];

    checkDictType( id, dictTypeUnknown );

    setFuncName( "MAdictUGetIthPOS" );

    sprintf( keyword, "%s %s", word, EPOSstr[epos] );

    // find contents
    r = dictFind( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    // found
    if( r == D_ERR_NO ) {
        cellTypeU *g; int posNum;

        posNum = dictResultLen / sizeof(cellTypeU);

        // check ith value is proper.
        if( posNum-1 < ith ) return FALSE; // return E_BIG_ITH;

        // set output argument
        g = (cellTypeU*) dictResult;
        *pPos = g[ith].pos;
        return TRUE;
    }

    // not found
    else if( r == D_NO_DATA )
        return FALSE; // return E_NO_WORD;
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

MAdictUGetFirstWord( id, s )
    int id;
    char *s;   // OUT
{
    register int i, j;
    char keyword[128];
    cellTypeU *g;
    int posNum;
    int r;

    checkDictType( id, dictTypeUnknown );

    setFuncName( "MAdictUGetFirstWord" );

    r = dictGetFirstWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    sprintf( s, "%s", keyword );

    // print POS following the keyword
    posNum = dictResultLen / sizeof(cellTypeU);
    g = (cellTypeU*) dictResult;

    for( i=0; i<posNum; i++ ) {     // CHANGE_CELLN
        strcat( s, " " );
        strcat( s, POSstr[g[i].pos] );
    }
    return TRUE;
}

MAdictUGetNextWord( id, s )
    int id;
    char *s;   // OUT
{
    register int i, j;
    char keyword[128];
    cellTypeU *g;
    int posNum;
    int r;

    checkDictType( id, dictTypeUnknown );

    setFuncName( "MAdictUGetNextWord" );

    r = dictGetNextWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    sprintf( s, "%s", keyword );

    // print POS following the keyword
    posNum = dictResultLen / sizeof(cellTypeU);
    g = (cellTypeU*) dictResult;

    for( i=0; i<posNum; i++ ) {     // CHANGE_CELLN
        strcat( s, " " );
        strcat( s, POSstr[g[i].pos] );
    }
    return TRUE;
}


/* I */

MAdictIInsert( id, word, irr )
    int id;
    char* word;
    int irr;
{
    register int i;
    int r;
    cellTypeI I;

    checkDictType( id, dictTypeIrregular );

    setFuncName( "MAdictIInsert" );

    // call dict API's find operation
    r = dictFind( id, word );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) {

        debugMesg("MAdictIInsert : insert Irregular %x\n", irr);

        I.irr = irr;

        // set global var.
        r = dictResultSet( &I, sizeof(cellTypeI) );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        // dictionary insert
        r = dictInsert( id, word );
        if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

        return TRUE;
    }
    else if(r == D_ERR_NO ) {   // data is existed and in global var.

        MAerror( NOT, E_EXIST_DATA, __FILE__, __LINE__,
             "MAdictIInsert : Data is already in dictionary.\n" );
        return FALSE;
    }
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

MAdictIDelete( id, word )
    int id;
    char* word;
{
    int r;

    checkDictType( id, dictTypeIrregular );

    setFuncName( "MAdictIDelete" );

    // delete 
    r = dictDelete( id, word );
    if( r < 0 ) {
        MAerror( NOT, E_DICT + r, __FILE__, __LINE__,
          "Word is not existed or delete operation is failed.\n");
        return FALSE;
    }
    return TRUE;
}

MAdictIGet( id, word, pIrr )
    int id;
    char* word;
    int *pIrr;    /* pointer to 4-byte bit-vector */
{
    register int i;
    int r;

    checkDictType( id, dictTypeIrregular );

    setFuncName( "MAdictIGet" );

    // find contents
    r = dictFind( id, word );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    // found
    if( r == D_ERR_NO ) {
        cellTypeI *g;

        // set output argument
        g = (cellTypeI*) dictResult;
        *pIrr = g->irr;
        return TRUE;
    }

    // not found
    else if( r == D_NO_DATA )
        return FALSE; // return E_NO_WORD;
    else
        MAerror( END, E_INTERNAL, __FILE__, __LINE__, "Internal Error.\n" );
}

MAdictIGetFirstWord( id, s )
    int id;
    char* s;   /* OUT */
{
    char keyword[128];
    char irrStr[80];
    cellTypeI *g;
    int r;

    checkDictType( id, dictTypeIrregular );

    setFuncName( "MAdictIGetFirstWord" );

    r = dictGetFirstWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    g = (cellTypeI*) dictResult;

    sprintf( s, "%s", keyword );

    sprintf( irrStr, " %x", g->irr );
    strcat( s, irrStr );
    return TRUE;
}

MAdictIGetNextWord( id, s )
    int id;
    char* s;   /* OUT */
{
    char keyword[128];
    char irrStr[80];
    cellTypeI *g;
    int r;

    checkDictType( id, dictTypeIrregular );

    setFuncName( "MAdictIGetFirstWord" );

    r = dictGetNextWord( id, keyword );
    if( r < 0 ) MAerror( END, E_DICT + r, __FILE__, __LINE__, dictErrorStr );

    if( r == D_NO_DATA ) return FALSE;

    g = (cellTypeI*) dictResult;

    sprintf( s, "%s", keyword );

    sprintf( irrStr, " %x", g->irr );
    strcat( s, irrStr );
    return TRUE;
}

struct mb {
    char *m; int irr;
};

static struct mb m2b[] = {
    { "b",     IRR_B    },    /* b is KTS */
    { "d",     IRR_D    },    /* d is KTS */
    { "geola", IRR_gela },    /* */
    { "h",     IRR_H    },    /* h is KTS */
    { "le",    IRR_l    },    /* this is very special marker */
    { "leo",   IRR_le   },    /* L is KTS */
    { "leu",   IRR_l_   },    /* l is KTS */
    { "neola", IRR_nela },    /* */
    { "s",     IRR_S    },    /* s is KTS */
    { "u",     IRR_u    },    /* */
    { NULL,    0        }
};

MAdictIMarker2BitVector( marker, pIrr )
    char* marker;
    int* pIrr;      /* OUT */
{
    register int i;

    for( i=0; m2b[i].m != NULL; i++ ) {
        if( !strcmp( m2b[i].m, marker ) ) {
            *pIrr = m2b[i].irr;
            return TRUE;
        }
    }
    return FALSE;
}
MAdictIBitVector2Marker( irr, marker )
    int  irr;
    char* marker;  /* OUT */
{
    register int i;

    for( i=0; m2b[i].irr != 0; i++ ) {
        if( m2b[i].irr == irr ) {
            strcpy( marker, m2b[i].m );
            return TRUE;
        }
    }
    return FALSE;
}



/* misc -------------------------------------------------------*/

checkDictType( id, type )
    int id;
    enum dictType type;
{
    setFuncName( "checkDictType" );

    // check dictionary type
    if( id2type[id] != type )
        MAerror( END, E_DICT_TYPE, __FILE__, __LINE__,
          "dictionary type is %d and not %d type.\n", id2type[id], type );
}

str2pos( s )
    char *s;
{
    register int i;
    for( i=0; POSstr[i] != NULL; i++ )
        if( !strcmp( s, POSstr[i] ) ) return i;
    return -1;
}

str2epos( s )
    char *s;
{
    register int i;
    for( i=0; EPOSstr[i] != NULL; i++ )
        if( !strcmp( s, EPOSstr[i] ) ) return i;
    return -1;
}


/* EOF --------------------------------------------------------*/
