/* irregular table and functions
 *
 * create : 6.13 (Tue), bgjang@csone
 * modify :
 * 6.19 (Mon), bgjang@csone - irrMakeEdges()
 * 6.26 (Mon), bgjang@csone - change mainly to accomodate the 'I','A','F'
 *                           notation for pattern.
 * 6.26 (Mon), bgjang@csone - version 0.5 alpha
 * 95.7.18 - 31 (Mon), bgjang
 *    All irregular rules are summarized in tables, irrTbl?.
 *    For symbols or english matching, 'E' symbol in table is added and
 *    _irrExcludeSymEng(), _irrGetIthSymEng(), _irrChartInsert() is added.
 *
 *
 *------------------------------------------------------------------
 *
 * Tables in this file must be remaking to accommodate all
 * advancement by Sang-Ho, Lee in his KTS before this system.
 * To do, I must check irrtbl.h file in KTS, and summarize it
 * as a table. Surely, it must be documented.
 *
 */

#include <stdio.h>
#include "irrtbl.h"
#include "pos.h"
#include "common.h"
#include "MAchart.h"
#include "MAerror.h"
#include <kimmocode.h>

#define IRR_LEN      9

#ifndef TRUE
#define TRUE (1)
#endif
#ifndef FALSE
#define FALSE (0)
#endif

/* static functions declarations ---------------------------------*/
static int _irrPatternMatch(/* pattern, str */);
static void _irrExcludeSymEng(/* org, des */);
static int _irrPatternSubstitute(/* pattern, tranPattern, str, tranResult */);
static int _irrGetIthSymEng(/* s, ith, out */);
static void _irrChartInsert(/* id, headV, tailV, str, orgStr, epos */);



/* static variables ----------------------------------------------*/

static int symEngNum;  /* used in _irrChartInsert() and irrMakeEdges() */



/* in pattern matching, there can be cho, jung, jong notation
 * the next is that.
 * ex) "Ie" means that first chosung must be appeared and the next 'e'.
 */
#define PATTERN_CHO    'I'
#define PATTERN_JUNG   'A'
#define PATTERN_JONG   'F'
#define PATTERN_SYMENG 'E'   /* these characters are not used by kimmo code */
#define PATTERN_ALL    "IAF"


/* an irregulation can be thought that some pattern is
 * transformed to the other one with sync point. Sync point
 * is an vertex ID which is common to the morphological analysis
 * with irregulation and without one.
 *
 * We assume the followings.
 * 1. Only one sync point is enough.
 * 2. Only the ending of word have to be shared to two analysis.
 *
 * We checked almost irregular rules and assure the above two
 * assumption is good for the other processing, such as,
 * syntax analysis.
 */


struct irr {
    char* pattern;
    char* patternWithSync;
    char* transWithSync;
    int   kind;               /* used as 4 byte bit vector */
};


struct irr irrTbl2[] = {
    { "Ie",     "I.e",     "I_.e",    IRR__     },
    { "Ia",     "I.a",     "I_.a",    IRR__     },
    { "AN",     "A.N",     "AL.N",    IRR_L     },
    { "An",     "A.n",     "AL.n",    IRR_L     },
    { "AB",     "A.B",     "AL.B",    IRR_L     },
    { "Ao",     "A.o",     "AL.o",    IRR_L     },
    { "Ae",     "A.e",     "AS.e",    IRR_S     },
    { "Aa",     "A.a",     "AS.a",    IRR_S     },
    { "A_",     "A._",     "AS._",    IRR_S     },
    { "pe",     "p.e",     "pu.e",    IRR_u     },
    { "aN",     "a.N",     "aH.N",    IRR_H     },
    { "aB",     "a.B",     "aH.B",    IRR_H     },
    { "aL",     "a.L",     "aH.L",    IRR_H     },
    { "aM",     "a.M",     "aH.M",    IRR_H     },
    { "eN",     "e.N",     "eH.N",    IRR_H     },
    { "eB",     "e.B",     "eH.B",    IRR_H     },
    { "eL",     "e.L",     "eH.L",    IRR_H     },
    { "eM",     "e.M",     "eH.M",    IRR_H     },
    { "Ia",     "I.a",     "Ia.a",    IRR_a     },   /*  ->  */
    { "Ie",     "I.e",     "Ie.e",    IRR_a     },   /*  -> ++ */
    { "I8",     "I.8",     "IaH.e",   IRR_H     },
    { "I8",     "I.8",     "I8.e",    IRR_8     },
    { "I9",     "I.9",     "I9.e",    IRR_9     },
    { "h8",     "h.8",     "ha.e",    IRR_h8    },
    { "Aa",     "A.a",     "A.ia",    IRR_i     },
    { "Ea",     "E.a",     "E.ia",    IRR_i     },
    { "Ae",     "A.e",     "A.ie",    IRR_i     },
    { "Ee",     "E.e",     "E.ie",    IRR_i     },
    { "Ao",     "A.o",     "A.io",    IRR_i     },
    { "Eo",     "E.o",     "E.io",    IRR_i     },
    { NULL,     NULL,      NULL,      NULL      }
};

struct irr irrTbl3[] = {
    { "Asi",    "A.si",    "AL.si",   IRR_L     },
    { "Awe",    "Aw.e",    "AB.e",    IRR_B     },
    { "Awa",    "Aw.a",    "AB.a",    IRR_B     },   /*  -> + */
    { "AwN",    "Aw.N",    "AB.N",    IRR_B     },
    { "Awn",    "Aw.n",    "AB.n",    IRR_B     },
    { "AwL",    "Aw.L",    "AB.L",    IRR_B     },
    { "Awl",    "Aw.l",    "AB.l",    IRR_B     },
    { "AwB",    "Aw.B",    "AB.B",    IRR_B     },
    { "Awo",    "Aw.o",    "AB.o",    IRR_B     },
    { "AwM",    "Aw.M",    "AB.M",    IRR_B     },
    { "Awm",    "Aw.m",    "AB.m",    IRR_B     },
    { "Aue",    "Au.e",    "AB.e",    IRR_B     },
    { "Aua",    "Au.a",    "AB.a",    IRR_B     },
    { "AuN",    "Au.N",    "AB.N",    IRR_B     },
    { "Aun",    "Au.n",    "AB.n",    IRR_B     },
    { "AuL",    "Au.L",    "AB.L",    IRR_B     },
    { "Aul",    "Au.l",    "AB.l",    IRR_B     },
    { "AuB",    "Au.B",    "AB.B",    IRR_B     },
    { "Auo",    "Au.o",    "AB.o",    IRR_B     },
    { "AuM",    "Au.M",    "AB.M",    IRR_B     },
    { "Aum",    "Au.m",    "AB.m",    IRR_B     },
    { "ALe",    "AL.e",    "AD.e",    IRR_D     },
    { "ALa",    "AL.a",    "AD.a",    IRR_D     },
    { "AL_",    "AL._",    "AD._",    IRR_D     },
    { "yaN",    "ya.N",    "yaH.N",   IRR_H     },
    { "yaB",    "ya.B",    "yaH.B",   IRR_H     },
    { "yaL",    "ya.L",    "yaH.L",   IRR_H     },
    { "yaM",    "ya.M",    "yaH.M",   IRR_H     },
    { "yeN",    "ye.N",    "yeH.N",   IRR_H     },
    { "yeB",    "ye.B",    "yeH.B",   IRR_H     },
    { "yeL",    "ye.L",    "yeH.L",   IRR_H     },
    { "yeM",    "ye.M",    "yeH.M",   IRR_H     },
    { "Aye",    "Ay.e",    "Ai.e",    IRR_ye_   },
    { "Iye",    "Iy.e",    "Ii.e",    IRR_ye_   },
    { "Iwa",    "Iw.a",    "Io.a",    IRR_wa    },
    { "Iue",    "Iu.e",    "Iw.e",    IRR_we    },
    { "Ago",    "A.go",    "A.igo",   IRR_i     },
    { "Ego",    "E.go",    "E.igo",   IRR_i     },
    { "Agi",    "A.gi",    "A.igi",   IRR_i     },
    { "Egi",    "E.gi",    "E.igi",   IRR_i     },
    { "Ana",    "A.na",    "A.ina",   IRR_i     },
    { "Ena",    "E.na",    "E.ina",   IRR_i     },
    { "Ani",    "A.ni",    "A.ini",   IRR_i     },
    { "Eni",    "E.ni",    "E.ini",   IRR_i     },
    { "Ada",    "A.da",    "A.ida",   IRR_i     },
    { "Eda",    "E.da",    "E.ida",   IRR_i     },
    { "Ala",    "A.la",    "A.ila",   IRR_i     },
    { "Ela",    "E.la",    "E.ila",   IRR_i     },
    { "Aya",    "A.ya",    "A.iya",   IRR_i     },
    { "Eya",    "E.ya",    "E.iya",   IRR_i     },
    { "Asi",    "A.si",    "A.isi",   IRR_i     },
    { "Esi",    "E.si",    "E.isi",   IRR_i     },
    { "AeV",    "A.eV",    "A.ieV",   IRR_i     },
    { "EeV",    "E.eV",    "E.ieV",   IRR_i     },
    { "Aji",    "A.ji",    "A.iji",   IRR_i     },
    { "Eji",    "E.ji",    "E.iji",   IRR_i     },
    /* { "Ada",    "A.da",    "Ai.da",  IRR_  }, */  /* Ҵ -> ̴ */
    { NULL,     NULL,      NULL,      NULL      }
};

struct irr irrTbl4[] = {
    { "Awsi",   "Aw.si",   "AB.si",   IRR_B     },
    { "Ausi",   "Au.si",   "AB.si",   IRR_B     },
    { "eLwe",   "eLw.e",   "elB.e",   IRR_B     },
    { "eLwa",   "eLw.a",   "elB.a",   IRR_B     },
    { "eLwN",   "eLw.N",   "elB.N",   IRR_B     },
    { "eLwn",   "eLw.n",   "elB.n",   IRR_B     },
    { "eLwL",   "eLw.L",   "elB.L",   IRR_B     },
    { "eLwl",   "eLw.l",   "elB.l",   IRR_B     },
    { "eLwB",   "eLw.B",   "elB.B",   IRR_B     },
    { "eLwo",   "eLw.o",   "elB.o",   IRR_B     },
    { "eLwM",   "eLw.M",   "elB.M",   IRR_B     },
    { "eLwm",   "eLw.m",   "elB.m",   IRR_B     },
    { "eLue",   "eLu.e",   "elB.e",   IRR_B     },
    { "eLua",   "eLu.a",   "elB.a",   IRR_B     },
    { "eLuN",   "eLu.N",   "elB.N",   IRR_B     },
    { "eLun",   "eLu.n",   "elB.n",   IRR_B     },
    { "eLuL",   "eLu.L",   "elB.L",   IRR_B     },
    { "eLul",   "eLu.l",   "elB.l",   IRR_B     },
    { "eLuB",   "eLu.B",   "elB.B",   IRR_B     },
    { "eLuo",   "eLu.o",   "elB.o",   IRR_B     },
    { "eLuM",   "eLu.M",   "elB.M",   IRR_B     },
    { "eLum",   "eLu.m",   "elB.m",   IRR_B     },
    { "ALle",   "ALl.e",   "Al_.e",   IRR_l_    },
    { "ALla",   "ALl.a",   "Al_.a",   IRR_l_    },
    { "l_le",   "l_l.e",   "l_.e",    IRR_le    },
    { "l_la",   "l_l.a",   "l_.a",    IRR_le    },
    { "haye",   "hay.e",   "ha.e",    IRR_ye    },
    { "ANga",   "A.Nga",   "A.iNga",  IRR_i     },
    { "ENga",   "E.Nga",   "E.iNga",  IRR_i     },
    { "ANd9",   "A.Nd9",   "A.iNd9",  IRR_i     },
    { "ENd9",   "E.Nd9",   "E.iNd9",  IRR_i     },
    { "ANji",   "A.Nji",   "A.iNji",  IRR_i     },
    { "ENji",   "E.Nji",   "E.iNji",  IRR_i     },
    { "ALqa",   "A.Lqa",   "A.iLqa",  IRR_i     },
    { "ELqa",   "E.Lqa",   "E.iLqa",  IRR_i     },
    { "ALs9",   "A.Ls9",   "A.iLs9",  IRR_i     },
    { "ELs9",   "E.Ls9",   "E.iLs9",  IRR_i     },
    { "ABni",   "A.Bni",   "A.iBni",  IRR_i     },
    { "EBni",   "E.Bni",   "E.iBni",  IRR_i     },
    { "AgeN",   "A.geN",   "A.igeN",  IRR_i     },
    { "EgeN",   "E.geN",   "E.igeN",  IRR_i     },
    { "Ag9V",   "A.g9V",   "A.ig9V",  IRR_i     },
    { "Eg9V",   "E.g9V",   "E.ig9V",  IRR_i     },
    { "Anya",   "A.nya",   "A.inya",  IRR_i     },
    { "Enya",   "E.nya",   "E.inya",  IRR_i     },
    { "Ad_N",   "A.d_N",   "A.id_N",  IRR_i     },
    { "Ed_N",   "E.d_N",   "E.id_N",  IRR_i     },
    { "Amye",   "A.mye",   "A.imye",  IRR_i     },
    { "Emye",   "E.mye",   "E.imye",  IRR_i     },
    { "Ajyo",   "A.jyo",   "A.ijyo",  IRR_i     },
    { "Ejyo",   "E.jyo",   "E.ijyo",  IRR_i     },
    { NULL,     NULL,      NULL,      NULL      }
};

struct irr irrTbl5[] = {
    { "eLwsi",  "eLw.si",  "eLB.si",  IRR_B     },
    { "eLusi",  "eLu.si",  "eLB.si",  IRR_B     },
    { "onela",  "on.ela",  "o.ela",   IRR_nela  },
    { "Aguna",  "A.guna",  "A.iguna", IRR_i     },
    { "Eguna",  "E.guna",  "E.iguna", IRR_i     },
    { "Al8do",  "A.l8do",  "A.il8do", IRR_i     },
    { "El8do",  "E.l8do",  "E.il8do", IRR_i     },
    { "Alola",  "A.lola",  "A.ilola", IRR_i     },
    { "Elola",  "E.lola",  "E.ilola", IRR_i     },
    { "AsyeV",  "A.syeV",  "A.isyeV", IRR_i     },
    { "EsyeV",  "E.syeV",  "E.isyeV", IRR_i     },
    { NULL,     NULL,      NULL,      NULL      }
};

struct irr irrTbl6[] = {
    { "gagela", "gag.ela", "ga.ela",  IRR_gela  },
    { "nagela", "nag.ela", "na.ela",  IRR_gela  },
    { "jagela", "jag.ela", "ja.ela",  IRR_gela  },
    { "iSgela", "iSg.ela", "iS.ela",  IRR_gela  },
    { NULL,     NULL,      NULL,      NULL      }
};

struct irr irrTbl8[] = {
    { "AlyemyeN",  "A.lyemyeN",  "A.ilyemyeN",  IRR_i    },
    { "ElyemyeN",  "E.lyemyeN",  "E.ilyemyeN",  IRR_i    },
    { NULL,        NULL,         NULL,          NULL     }
};

struct irr *irrTblArray[IRR_LEN+1] =
       { NULL, NULL, irrTbl2, irrTbl3, irrTbl4, irrTbl5, irrTbl6, NULL, irrTbl8, NULL };


int irrIsPatternIrr( kimmo )
    char* kimmo;
{
    register int i;
    char s[128];
    struct irr *pIrr;

    /* exclude symbol/english and convert it to one character */
    _irrExcludeSymEng( kimmo, s );

    if( strlen(s) > IRR_LEN ) return FALSE;

    pIrr = irrTblArray[strlen(s)];
    if( pIrr == NULL ) return FALSE;

    for( i=0; pIrr[i].pattern != NULL; i++ ) {
        if( _irrPatternMatch( pIrr[i].pattern, s ) ) return TRUE;
    }
    return FALSE;
}

/* substr is sub string of word which can be transformed to
 * the other sub string by irregular rule.
 *
 * And for one sub string, there may be more than two irregular
 * rules. So this functions must return FALSE for the case that there is
 * no more rule to be applied, else return TRUE.
 *
 * And if this functions is called by substr is NULL.
 */
int irrMakeEdges( chartID, ith, headV, tailV, substr, hStackID, pIrrKind, pMidV )
    int chartID;
    int ith;
    int headV, tailV;
    char *substr;
    int hStackID;
    int *pIrrKind;       /* OUT : 4 byte bit-vector */
    int *pMidV;          /* OUT : mid vertex */
{
    register int i, j;
    int r;
    struct irr *pIrr;
    char pws[64];
    char tws[64];
    char s[128];
    int hv, tv, idInterval;
    enum EPOS epos;

    /* exclude symbol/english and convert it to one character */
    _irrExcludeSymEng( substr, s );
    symEngNum = 0;

    if( strlen(s) > IRR_LEN ) return FALSE;

    pIrr = irrTblArray[strlen(s)];
    if( pIrr == NULL ) return FALSE;

    for( i=0; pIrr[i].pattern != NULL; i++ ) {
        if( _irrPatternMatch( pIrr[i].pattern, s ) ) {
            ith--;

            /* if there is matched one, then check
             * patternWithSync, transWithSync and make sync (mid vertex ID)
             * between original pattern and transformed one. And
             * insert result edges in chart.
             *
             * and return TRUE.
             */
            if( ith < 0 ) {
                int midV, a;
                char *p;
                char *twsHead, *twsTail;
                char oneChar[2];

                strcpy( pws, pIrr[i].patternWithSync );
                _irrPatternSubstitute( pIrr[i].pattern, pIrr[i].transWithSync,
                                       s, tws );
                *pIrrKind = pIrr[i].kind;

                a = strlen(pws);
                p = (char*) strtok( pws, "." );

                /* calculate the mid vertex(sync point) ID 
                 * independent the interval of each eage's
                 * head and tail vertex ID.
                 */
                // midV = (tailV - headV) * strlen(p) / a;
                midV = headV + VII * strlen(p);

                twsHead = (char*) strtok( tws, "." );
                twsTail = (char*) strtok( NULL, "." );

                /* before sync point
                 */
                idInterval = (midV - headV)/strlen(twsHead);
                for( j=0; j < strlen(twsHead); j++ ) {
                    hv = headV + j * idInterval;
                    tv = (j == strlen(twsHead)-1) ? midV : hv + idInterval;

                    /* if this edge's tail vertex ID is sync point's one
                     * then set epos as QP, so normal searching only the
                     * front portion.
                     */
                    epos = (j == strlen(twsHead)-1) ? QP : QQ;

                    oneChar[0] = twsHead[j]; oneChar[1] = '\0';
                    // MAchartInsert( chartID, hv, tv, oneChar, epos, -1.0, &e );
                    _irrChartInsert( chartID, hv, tv, oneChar, substr, epos );

                    r = isPush( hStackID, hv );
                    if( r == FALSE )
                        MAerror( END, E_ISTACKPUSH, __FILE__, __LINE__,
                          "Push op in integer stack is failed. full.\n" );
                }

                /* after sync point
                 */
                idInterval = (tailV - midV)/strlen(twsTail);
                for( j=0; j < strlen(twsTail); j++ ) {
                    hv = midV + j * idInterval;
                    tv = (j == strlen(twsTail)-1) ? tailV : hv + idInterval;

                    /* If this edge's head vertex ID is sync point's
                     * vertex ID, then set epos as QB, so normal
                     * searching only the back portion.
                     */
                    epos = (j == 0) ? QB : QQ;

                    oneChar[0] = twsTail[j]; oneChar[1] = '\0';
                    // MAchartInsert( chartID, hv, tv, oneChar, epos, -1.0, &e );
                    _irrChartInsert( chartID, hv, tv, oneChar, substr, epos );

                    r = isPush( hStackID, hv );
                    if( r == FALSE )
                        MAerror( END, E_ISTACKPUSH, __FILE__, __LINE__,
                          "Push op in integer stack is failed. full.\n" );
                }

                *pMidV = midV;
                return TRUE;
            }

        }
    }
    return FALSE;
}

static int _irrPatternMatch( pattern, str )
    char *pattern;
    char *str;
{
    register int i;
    if( strlen(str) != strlen(pattern) ) return FALSE;

    for( i=0; i < strlen(str); i++ ) {
        switch( pattern[i] ) {
        case PATTERN_CHO :
            if( !strchr(Is_cho, str[i]) ) return FALSE;
            break;
        case PATTERN_JUNG :
            if( !strchr(Is_jung, str[i]) ) return FALSE;
            break;
        case PATTERN_JONG :
            if( !strchr(Is_jong, str[i]) ) return FALSE;
            break;

        /* default action include PATTERN_SYMENG match
         */
        default :
            if( pattern[i] != str[i] ) return FALSE;
        }
    }
    return TRUE;
}


/* example says all of this function's role
 */
static int _irrPatternSubstitute( pattern, tranPattern, str, tranResult )
    char *pattern;      /* IN  : ex. "Asi" */
    char *tranPattern;  /* IN  : ex. "AL.si" */
    char *str;          /* IN  : ex. "osi" */
    char *tranResult;   /* OUT : ex. "oL.si" */
{
    register int i, j, bool;
    if( strlen(str) != strlen(pattern) ) return FALSE;

    for( i=0, j=0; i < strlen(pattern); i++ ) {
        if( strchr( PATTERN_ALL, pattern[i] ) ) {
            bool = 0;
            for( ; j < strlen(tranPattern); j++ ) {
                if( tranPattern[j] == pattern[i] ) {
                    tranResult[j++] = str[i];
                    bool = 1;
                    break;
                }
                tranResult[j] = tranPattern[j];
            }
            if( bool == 0 ) return FALSE;
        }
    }
    for( ; j < strlen(tranPattern); j++ ) {
        if( strchr( PATTERN_ALL, tranPattern[j] ) ) return FALSE;
        tranResult[j] = tranPattern[j];
    }
    tranResult[j] = '\0';
    return TRUE;
}

static void _irrExcludeSymEng( org, des )
    char *org, *des;
{
    register int i,j;

    /* if the kimmo string contains the symbol or english enclosed
     * by ^N, or ^O, then it convert that part to one character
     * PATTERN_SYMENG ('E').
     */
    for( i=0, j=0; i<strlen(org); i++, j++ ) {
        if( org[i] == CTRL(N) ) {
            while( org[++i] != CTRL(O) ) ; /* skip all sym/eng characters */
            des[j] = PATTERN_SYMENG;
        }
        else
            des[j] = org[i];
    }
    des[j] = '\0';
}

/* get the ith symbol/english strings from s
 * if there is one, then return TRUE with output in out
 * otherwise, return FALSE
 */
static int _irrGetIthSymEng( s, ith, out )
    char *s;
    int ith;
    char *out;   /* OUT : ith symbol/english string will be in here */
{
    register int i,j;

    for( i=0,j=0; i<strlen(s); i++ ) {
        if( s[i] == CTRL(N) ) {
            ith--;
            if( ith < 0 ) {
                do {
                    out[j++] = s[i++];
                } while( s[i] != CTRL(O) );
                out[j++] = CTRL(O);
                out[j] = '\0';

                return TRUE;
            }
        }
    }
    return FALSE;
}

/* if PATTERN_SYMENG is in the currently inserting edge, then
 * change it to original string and insert
 */
static void _irrChartInsert( id, headV, tailV, oneChar, orgStr, epos )
    int id;
    int headV, tailV;
    char *oneChar;
    char *orgStr;
    enum EPOS epos;
{
    int r;
    char orgSymEngStr[128];
    int e;

    if( oneChar[0] == PATTERN_SYMENG ) {
        r = _irrGetIthSymEng( orgStr, symEngNum, orgSymEngStr );
        if( r == FALSE )
            MAerror( END, E_INTERNAL, __FILE__, __LINE__,
                "symbol/english string manipulation error in irregular handling.\n" );
        MAchartInsert( id, headV, tailV, orgSymEngStr, epos, -1.0, &e );
    }
    else
        MAchartInsert( id, headV, tailV, oneChar, epos, -1.0, &e );

    symEngNum++;
}
