/* pos.h (header file for part of speech)
 *
 * create : 95.6.4 (Sun), bgjang@csone
 * modify :
 * 95.6.13 (Tue), bgjang@csone - add EPOS enumeration type
 * 95.7.31 (Mon), bgjang - add uf, usy, unnn to POS and EPOS
 *
 *
 */
#ifndef __POS_H__
#define __POS_H__

/* POS (Part of Speech)
 *
 * Important Note :
 *
 * 1. INI is needed to check whether a POS can be appeared in the head of
 *    word or not.
 * 2. FIN is needed to check whether a POS can be appeared in the tail of
 *    word or not.
 *
 * 3. sp (space character) is originally included in a POS in
 *    Je-Hoom, Kim's paper but sp have a role of delimiter or words
 *    in one sentence in this program, so sp is deleted in POS set
 *    and connectivity table too.
 *
 *    original :
 *    sp : space character
 *    sc, ss, sl, sr, sm, su, sw, sp, sy,                  // 9
 *    "s,", "s.", "s`", "s'", "s-", "su", "sw", "sp", "sy", in POSstr
 *
 *    here :
 *    as you can see in the following real code and MAdict.c file.
 *
 * 4. INT : mean _ in kimmon code and
 *          int in KTS (Korean Tagging System) by Sang-Ho, Lee
 *
 * 5. uf, usy, unnn is not included in POS but it is need to
 *    represent foreign/symbol/number which cannot be recorded
 *    in normal dictionary.
 *         
 *
 * EPOS (Extended POS)
 *
 * 1. It have no relations with the tagging set. It is only for
 *    procedure to search normal dictionary. It is same with POS
 *    except addition of QP, QB and QQ to POS.
 *
 * 2. Q means "Question mark". QP for "?P", QB for "?B" and QQ for "??".
 *    Each means that "search only Previous part (this syllable(?) can
 *    be attached to only Previous part)", "search only Back part" and
 *    "This syllable(?) can be attached to anywhere".
 *
 */

/* default frequency of which word is not existed in normal dictionary
 * this must be non-zero value.
 */
#define DEFAULT_FREQ     1.0

#define NUM_POS  58   /* including INI, FIN, INT, usy, uf, unnn */

enum POS {
     INI,                                                 /*  0 */
     sc, ss, sl, sr, sm, su, sw, sy,                      /*  8 */
     f,                                                   /*  9 */
     nnn, nct, nca, ncs, nc, nq, nbu, nb, npp, npd, nn,   /* 20 */
     pv, pad, pa, px,                                     /* 24 */
     md, mn, m,                                           /* 27 */
     at, ad, ajw, ajs, a,                                 /* 32 */
     i,                                                   /* 33 */
     jc, jcm, jcv, jca, jcp, jx, jj,                      /* 40 */
     ecq, ecs, ecx, exm, exn, exa, efp, ef,               /* 48 */
     xn, xpv, xpa, xa,                                    /* 52 */
     FIN,                                                 /* 53 */
     INT, usy, uf, unnn };


#define NUM_EPOS 58  /* including QP, QB, QQ to POS */

enum EPOS {
     INI,                                                 /*  0 */
     sc, ss, sl, sr, sm, su, sw, sy,                      /*  8 */
     f,                                                   /*  9 */
     nnn, nct, nca, ncs, nc, nq, nbu, nb, npp, npd, nn,   /* 20 */
     pv, pad, pa, px,                                     /* 24 */
     md, mn, m,                                           /* 27 */
     at, ad, ajw, ajs, a,                                 /* 32 */
     i,                                                   /* 33 */
     jc, jcm, jcv, jca, jcp, jx, jj,                      /* 40 */
     ecq, ecs, ecx, exm, exn, exa, efp, ef,               /* 48 */
     xn, xpv, xpa, xa,                                    /* 52 */
     FIN,                                                 /* 53 */
     INT, usy, uf, unnn,                                  /* 54 */
     QP, QB, QQ, NONE };                                  /* 58 */
     

/* This strings are for printing POS, EPOS in human-readible form.
 * defined currently in MAdict.c
 */
extern char *POSstr[];  
extern char *EPOSstr[];

#endif
