/* token.c
 *
 * create : 95.5.30 (Tue), bgjang@csone
 * modify :
 * 95.6.1 (Fri), bgjang@csone - tokenSetInput() is inserted.
 * 95.6.2 (Fri), bgjang@csone - fix token type in last
 *
 */ 

#include <stdio.h>
#include <lex.h>
#include <token.h>
#include <MAerror.h>

/* for lex ----------------------------------------------------------*/

extern FILE* yyin;


/* global variable --------------------------------------------------*/

char *tokenTypeStr[] = {"T_KOR", "T_SYM", "T_ENG", "T_NUM"};

aToken *tokenList;          // list of tokens
int tokenNumber;            // number of current tokens
static int tokenAllocated;  // number of tokens allocated

char tokenErrorStr[256];
unsigned char lexResult[LEN_LEXRESULT];        // yylex() will print its result in here.



/* main coding -----------------------------------------------------*/

tokenInit()
{
    tokenList = (aToken*) malloc( sizeof(aToken) * INIT_TOKEN );
    if( tokenList == NULL ) {
        sprintf(tokenErrorStr, "malloc() is failed.\n");
        return T_ERR_MALLOC;
    }
    tokenNumber = 0;
    tokenAllocated = INIT_TOKEN;
    return T_ERR_NO;
}

void tokenDestroy()
{
    free( tokenList );
}

// set input file pointer
void tokenSetInput( fp )
    FILE* fp;
{
    yyin = fp;
}

// tokenization
// and make list of token and store in tokenList.
// for one sentences (which is delimited by newline character)
//
// 1 : there is more stuff to be tokenized
// 0 : no data is left to tokenization 
//
tokenMake()
{
    int r;

    strcpy( lexResult, "" );

    // initialize the token global variables and newly make
    // list of token.
    // But once realloc memory area will not be shirinked.
    //
    tokenNumber = 0;

    r = yylex();
    if( r < 0 ) {
        sprintf( tokenErrorStr, "lex output size is over the maximum value LEN_LEXRESULT.\n");
        return T_ERR_LEXOUTPUTSIZE;
    }

    // after lex, we must delete the last spaces in lexResult
    // array not to cause the side effect when we use
    // _tokenSplitBySpace()

    delspace( lexResult );
    debugMesg("\nlexResult : %s\n", lexResult);

    if( r == 1 ) {  // new line is accepted
        r = _tokenMake();
        if( r < 0 ) return r;
        return 1;
    }

    // return when tokenization is ended.
    else if( r == 0 ) return 0;
}

// lexResult -> list of token data structure
_tokenMake()
{
    char contents[MAX_TOKEN_LEN];
    char type[16];
    unsigned char *p;
    int r;

    // _tokenSplitBySpace() functions first argument must be
    // some char* which pointing to the variable to be processed
    // when it is firstly called.
    // after that, first argument must be NULL to processing
    // the lefted part of variables
    //
    p = lexResult;

    while(1) {
        // get one token
        r = _tokenSplitBySpace( p, contents, MAX_TOKEN_LEN );
        p = NULL;
        if( r < 0 ) return r;
        if( r == T_NO_DATA ) break;

        // get type of this token
        r = _tokenSplitBySpace( NULL, type, 16 );
        if( r < 0 ) return r;
        if( r == T_NO_DATA ) {
            sprintf(tokenErrorStr, "Internal Error. Cannot find a pair in lex output form.\n");
            return T_ERR_LEXOUTPUT;
        }

        // insert one node
        r = _tokenInsertOneNode( type, contents );
        if( r < 0 ) return r;
    }
    return T_ERR_NO;
}

_tokenStr2enum( s )
    char* s;
{
    register int i;

    for( i=0; i<NUM_TOKENTYPE; i++ ) {
        if( !strcmp( tokenTypeStr[i], s ) ) return i;
    }
    return -1;
}

_tokenInsertOneNode( type, contents )
    char* type;
    char* contents;
{
    int ttt;   // temporary token type
    int r;

    ttt = _tokenStr2enum( type );

    if( ttt == -1 ) {
        sprintf(tokenErrorStr, "Internal Error. Token type is wrong.\n");
        return T_ERR_TOKENTYPE;
    }

    if( tokenNumber == tokenAllocated ) {
        r = _tokenRealloc();
        if( r < 0 ) return r;
    }

    tokenList[tokenNumber].tt = (enum tokenType) ttt;

    // in case korean, convert it to kimmo code and store
    if( tokenList[tokenNumber].tt == tokenK ) {
        char contentsKimmo[MAX_TOKEN_LEN*2];

        ks2kimmo( contents, contentsKimmo );
        if( strlen(contentsKimmo) >= MAX_TOKEN_LEN ) {
            sprintf("Converted kimmo code's length is too big, so not fit to current token length. Change MAX_TOKEN_LEN.\n");
            return T_ERR_MAXTOKENLEN;
        }
        strcpy( tokenList[tokenNumber].string, contentsKimmo );
    }
    else
        strcpy( tokenList[tokenNumber].string, contents );

    tokenNumber++;

    return T_ERR_NO;
}

_tokenSplitBySpace( s, r, maxLen )
    unsigned char* s;
    char* r;
    int maxLen;
{
    register int i;
    static unsigned char line[LEN_LEXRESULT];
    static unsigned char *p;

    if( s == NULL ) {
        if( p == NULL ) return T_NO_DATA;

        for( i=0; *(p+i) != '\0'; i++ ) {
            if( i >= maxLen-1 ) {
                sprintf(tokenErrorStr, "Max Length of result array is exceeded.\n" );
                return T_ERR_MAXTOKENLEN;
            }
            if( i == 0 || p[i] != ' ' ) r[i] = p[i]; 
            else break;
        }
        r[i] = '\0';
        if( *(p+i) != '\0' ) p = p + (i+1);
        else {
            p = NULL;
            strcpy( line, "" );
        }
        return T_ERR_NO;
    }
    else {
        debugMesg("_tokenSplitBySpace : s is not NULL.\n", s);
        strcpy( line, s );
        p = line;

        if( *p == '\0' ) return T_NO_DATA;

        for( i=0; *(p+i) != '\0'; i++ ) {
            if( i >= maxLen-1 ) {
                sprintf(tokenErrorStr, "Max Length of result array is exceeded.\n" );
                return T_ERR_MAXTOKENLEN;
            }
            if( i == 0 || p[i] != ' ' ) r[i] = p[i]; 
            else break;
        }
        r[i] = '\0';
        if( *(p+i) != '\0' ) p = p + (i+1);
        else p = NULL;
        return T_ERR_NO;
    }
}

// dump list of tokens
void tokenDump()
{
    register int i;
    char stringKS[MAX_TOKEN_LEN];

    printf("Token List Dump ---- BEGIN\n");
    printf("TYPE   contents\n");

    for( i=0; i<tokenNumber; i++ ) {
        printf("%6s %s",
          tokenTypeStr[tokenList[i].tt], tokenList[i].string);

        // in case korean, then print kimmo code and ks code together
        if( tokenList[i].tt == tokenK ) {
            kimmo2ks( tokenList[i].string, stringKS );
            printf(" %s\n", stringKS );
        }
        else
            printf("\n");
    }

    printf("Token List Dump ---- END\n");
}


/* misc ------------------------------------------------------------*/

_tokenRealloc()
{
    // increase 50%
    tokenAllocated += tokenAllocated/2;

    tokenList = (aToken*) realloc( (char*) tokenList,
                                   sizeof(aToken) * tokenAllocated );
    if( tokenList == NULL ) {
        sprintf(tokenErrorStr, "realloc() is failed.\n");
        return T_ERR_REALLOC;
    }
    debugMesg("token memory is reallocated, %d elments now.\n", tokenAllocated);
    return T_ERR_NO;
}

// delete spaces in the last of strings
// "run run  " -> "run run"
// " " -> ""
//
delspace( s )
    char* s;
{
    char*   c;
    for( c = s + strlen(s) - 1; *c == ' '; ) {
        *c = '\0';
        if(c == s) break;
        c--;
    }
}
