/* dictionary management tool
 *
 * create : 95.5.27 (Sat), bgjang@csone
 * modify :
 * 95.5.28 (Sun), bgjang@csone - can handle normal type dictionary well
 * 95.5.29 (Mon), bgjang@csone - -v option/command is inserted.
 * 95.6.2 (Fri), bgjang@csone - normal dict, SI(^N)/SO(^O) supporting
 * 95.6.6 (Tue), bgjang@csone - exceptional dictionary
 * 95.6.7 (Wed), bgjang@csone - exceptional dictionary complete without
 *                              delete command
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <limits.h>
#include "../common.h"
#include "../pos.h"
#include <dictAPI.h>
#include <MAdict.h>
#include <token.h>
#include "dictman.h"
#include <assert.h>

/* local definitions -------------------------------------------------*/

// readFrom value can have the three value
#define RF_STDIN   1
#define RF_FILE    0
#define RF_NONE    -1

//
// error code
// Modify Note :
// 1. insert here error Code
// 2. modify printError() function
//
#define ENODICT       -1
#define EUNKNOWNCOM   -2 
#define EFORMAT       -3
#define EDICTTYPE     -4
#define EMANYARG      -5
#define ENOTFOR       -6
#define EPOSVAL       -7
#define ENOTOPENED    -8
#define EIRRMARKER    -9


/* global variables -------------------------------------------------*/
int  readFrom = RF_NONE;      // read from standard input or file
char inFn[_POSIX_PATH_MAX];   // input file path name when readFrom is RF_FILE
int  dOpend = FALSE;          // dictionary opend ? TRUE or FALSE
char dFn[_POSIX_PATH_MAX];    // dictionary path name
int  dt = dictTypeNone;       // dictionary type
int  ignoreErr = FALSE;       // ignore error when error is occured.

int dictID;                   // dictionary ID


/* main code ---------------------------------------------------------*/

main( argc, argv )
    int argc;
    char *argv[];
{
    register int i;
    char line[512], lineKimmo[512], s[512];
    FILE* fp;
    int r;
    int lineNo = 0;

    // initialize global variables if needed.
    //
    MAdictInit();
    tokenInit();

    // handling arguments
    //
    if( argc == 1 ) {
        printUsage( argv[0] );
        exit(1);
    }

    if( handleOptions( argc, argv ) != TRUE ) exit(1);

    debugMesg("Options are processed correctly.\n");

    // open data file if needed
    if( readFrom == RF_FILE ) {
        fp = fopen( inFn, "r" );
        debugMesg("file is opened with read permission only.\n");
        if( fp == NULL ) {
            fprintf(stderr, "cannot open file %s.\n", inFn);
            exit(1);
        } 
        tokenSetInput( fp );
    }
    
    // read a line and processing it.
    //
    while(1) {

        if( readFrom == RF_FILE ) {
            if( fgets( s, 512, fp ) == NULL ) break;
            s[strlen(s)-1] = '\0';

            if( lineNo % 200 == 0 && lineNo != 0 ) printf(".");
        }
        else if( gets( s ) == NULL ) break;

        lineNo++;

        if( s[0] == 'T' ) {

            // read a line and make list of tokens
            r = tokenMake();
            if( r < 0 ) {
                printf("%d line\n%s", lineNo, tokenErrorStr );
                tokenDestroy();
                exit(1);
            }
            lineNo++;

            // if( r != 0 ) tokenDump();
            if( r == 0 ) break;

            // processing
            //
            r = processAfterTokenization();
        }
        else
            r = processOneLine( s );

        if( r < 0 ) {
            printError( r );

            // in case file, print the line number and contents.
            if( readFrom == RF_FILE )
            //    fprintf(stderr, "%d line : %s\n", lineNo, line );
                fprintf(stderr, "%d line\n", lineNo );

            // if -i options is specified, then ignore errors
            if( ignoreErr != TRUE ) exit(1);
        }
    } // of while

    // destroy list of token
    tokenDestroy();

    // close opened file and dictionary
    if( readFrom == RF_FILE ) fclose(fp);
    if( dOpend == TRUE ) MAdictClose(dictID);

    exit(1);
}

/* processing one line --------------------------------------------------*/


/*-----------------------------------------------------------------------
 * command list of dictionary manager
 *
    word : TRUE

    all    : D dictionary-name dictionary-type
           : C
           : V

    normal : IP word POS      -> obsolete, 95.5.31       token
           : IP word POS freq -> new, 95.5.31            token
           : DP word POS                                 token
           : ZW word                                     token
           : LW word                                     token

    exce   : ID lm+word+rm data-string
  !          DD lm+word+rm ith
             LD lm+word+rm
             ED lm+word+rm

   unknown : IU word/EPOS POS
             DU word/EPOS POS
             ZU word/EPOS
             LU word/EPOS

 irregular : II word irr-marker
             DI word
             LI word


            
 *
 *------------------------------------------------------------------------
 */

processOneLine( s )
    char *s;
{
    char line[512];
    char *arg[5];
    int r;
    char lm, rm;
    char word[128], wordKimmo[128], argKimmo[256];
    char str[1024];
    enum EPOS epos;
    enum POS pos;

    strcpy( line, s );

    arg[0] = (char*) strtok( line, " \t" );

    // D dictionary-name dictionary-type
    //
    if( !strcmp( arg[0], "D" ) ) {
        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] == NULL ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] == NULL ) return EFORMAT;
        arg[3] = (char*) strtok( NULL, " \t" ); if( arg[3] != NULL ) return EMANYARG;

        // set dictionary type and dictionary name
        if( !strchr( "neui", arg[2][0] ) ) return EDICTTYPE;

        switch( arg[2][0] ) {
        case 'n' : dt = dictTypeNormal; break;
        case 'e' : dt = dictTypeExce; break;
        case 'u' : dt = dictTypeUnknown; break;
        case 'i' : dt = dictTypeIrregular; break;
        }

        strcpy( dFn, arg[1] );

        // if one dictionary is opend already, then close it.
        if( dOpend == TRUE ) {
            MAdictClose(dictID);
            dOpend = FALSE;
            debugMesg("opened dictionary is closed.\n");
        }

        dictID = MAdictOpen( dFn, O_RDWR | O_CREAT, 0644, dt );
        dOpend = TRUE;

        debugMesg("currently opend dict ID is %d.\n", dictID);
    }

    // C
    //
    else if( !strcmp( arg[0], "C" ) ) {
        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] != NULL ) return EMANYARG;

        if( dOpend != TRUE ) return ENOTOPENED;

        MAdictClose(dictID);
        dOpend = FALSE;
        debugMesg("closed operations is succeeded.\n");
    }

    // V
    //
    else if( !strcmp( arg[0], "V" ) ) {
        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] != NULL ) return EMANYARG;

        if( dOpend != TRUE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;

        if( dt == dictTypeNormal ) {
            int h, j; char line[256], stripline[256];

            for( h = MAdictNGetFirstWord( dictID, line );
              h != FALSE; h = MAdictNGetNextWord( dictID, line ) ) {

                assert( strlen(line) < 256 );

                MAdictNKey2Kimmo( line, stripline );
                printf("%s\n", stripline );
            }
        }
        else if( dt == dictTypeUnknown ) {
            int h, j; char line[256];

            for( h = MAdictUGetFirstWord( dictID, line );
              h != FALSE; h = MAdictUGetNextWord( dictID, line ) ) {

                assert( strlen(line) < 256 );

                printf("%s\n", line );
            }
        }
        else if( dt == dictTypeIrregular ) {
            int h, j; char line[256], stripline[256];

            for( h = MAdictIGetFirstWord( dictID, line );
              h != FALSE; h = MAdictIGetNextWord( dictID, line ) ) {

                assert( strlen(line) < 256 );

                printf("%s\n", line );
            }
        }
        else {
            int h, j; char line[512]; 

            for( h = MAdictEGetFirstWord( dictID, line );
              h != FALSE; h = MAdictEGetNextWord( dictID, line ) ) {

                assert( strlen(line) < 512 );

                printf("%s\n", line );
            }
        }
    }

    // ID command
    //
    else if( !strcmp( arg[0], "ID" ) ) {

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeExce ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] == 0 ) return EFORMAT;
        arg[3] = (char*) strtok( NULL, " \t" ); if( arg[3] != 0 ) return EMANYARG;

        lm = (char) arg[1][0];
        rm = (char) arg[1][strlen(arg[1])-1];

        strncpy( word, arg[1]+1, strlen(arg[1]) - 2 );
        word[strlen(arg[1]) - 2] = '\0';
        ks2kimmo( word, wordKimmo );
        ks2kimmo( arg[2], argKimmo );

        MAdictEInsertData( dictID, wordKimmo, lm, rm, argKimmo );
    }

    // LD command
    //
    else if( !strcmp( arg[0], "LD" ) ) {

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeExce ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] != 0 ) return EMANYARG;

        lm = arg[1][0];
        rm = arg[1][strlen(arg[1])-1];

        strncpy( word, arg[1]+1, strlen(arg[1]) - 2 );
        word[strlen(arg[1]) - 2] = '\0';
        ks2kimmo( word, wordKimmo );

        r = MAdictEGetData( dictID, wordKimmo, lm, rm, str );
        if( r == TRUE )
            printf("%s\n", str );
    }

    // ED command
    //
    else if( !strcmp( arg[0], "ED" ) ) {

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeExce ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] != 0 ) return EMANYARG;

        lm = arg[1][0];
        rm = arg[1][strlen(arg[1])-1];

        strncpy( word, arg[1]+1, strlen(arg[1]) - 2 );
        word[strlen(arg[1]) - 2] = '\0';
        ks2kimmo( word, wordKimmo );

        r = MAdictEExistData( dictID, wordKimmo, lm, rm );
        if( r == TRUE )
            printf("Exist\n" );
        else
            printf("Not Exist\n" );
    }

    // IU word/EPOS POS
    //
    else if( !strcmp( arg[0], "IU" ) ) {
        
        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeUnknown ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, "/" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] == 0 ) return EFORMAT;
        arg[3] = (char*) strtok( NULL, " \t" ); if( arg[3] == 0 ) return EFORMAT;
        arg[4] = (char*) strtok( NULL, " \t" ); if( arg[4] != 0 ) return EMANYARG;

        epos = str2epos( arg[2] ); if( epos == -1 ) return EPOSVAL;
        pos = str2pos( arg[3] );   if( pos == -1 ) return EPOSVAL;
        ks2kimmo( arg[1], wordKimmo );
        MAdictUInsertUnknown( dictID, wordKimmo, epos, pos );
    }

    // DU word/EPOS POS
    //
    else if( !strcmp( arg[0], "DU" ) ) {
        enum EPOS epos;
        enum POS pos;
        
        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeUnknown ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, "/" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] == 0 ) return EFORMAT;
        arg[3] = (char*) strtok( NULL, " \t" ); if( arg[3] == 0 ) return EFORMAT;
        arg[4] = (char*) strtok( NULL, " \t" ); if( arg[4] != 0 ) return EMANYARG;

        epos = str2epos( arg[2] ); if( epos == -1 ) return EPOSVAL;
        pos = str2pos( arg[3] );   if( pos == -1 ) return EPOSVAL;
        ks2kimmo( arg[1], wordKimmo );
        MAdictUDeletePOS( dictID, wordKimmo, epos, pos );
    }

    // ZU word/EPOS
    else if( !strcmp( arg[0], "ZU" ) ) {
        enum EPOS epos;
        enum POS pos; int h, i;
        
        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeUnknown ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, "/" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] == 0 ) return EFORMAT;
        arg[3] = (char*) strtok( NULL, " \t" ); if( arg[3] != 0 ) return EMANYARG;

        epos = str2epos( arg[2] ); if( epos == -1 ) return EPOSVAL;
        ks2kimmo( arg[1], wordKimmo );
        MAdictUZap( dictID, wordKimmo, epos );
    }

    // LU word/EPOS
    else if( !strcmp( arg[0], "LU" ) ) {
        enum EPOS epos;
        enum POS pos; int h, i;
        
        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeUnknown ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, "/" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] == 0 ) return EFORMAT;
        arg[3] = (char*) strtok( NULL, " \t" ); if( arg[3] != 0 ) return EMANYARG;

        epos = str2epos( arg[2] ); if( epos == -1 ) return EPOSVAL;
        ks2kimmo( arg[1], wordKimmo );
        for( h = TRUE, i=0; h == TRUE ; i++ ) {
            h = MAdictUGetIthPOS( dictID, wordKimmo, epos, i, &pos );
            if( h == TRUE ) printf("%s ", POSstr[pos]);
        }
        if( i > 1 ) printf("\n");
    }

    // II word irr-marker
    else if( !strcmp( arg[0], "II" ) ) {
        int irr;
        
        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeIrregular ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] == 0 ) return EFORMAT;
        arg[3] = (char*) strtok( NULL, " \t" ); if( arg[3] != 0 ) return EMANYARG;

        r = MAdictIMarker2BitVector( arg[2], &irr );
        if( r == FALSE ) return EIRRMARKER;
        ks2kimmo( arg[1], wordKimmo );
        MAdictIInsert( dictID, wordKimmo, irr );
    }

    // DI word
    else if( !strcmp( arg[0], "DI" ) ) {

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeIrregular ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] != 0 ) return EMANYARG;

        ks2kimmo( arg[1], wordKimmo );

        MAdictIDelete( dictID, wordKimmo );
    }

    // LI word
    else if( !strcmp( arg[0], "LI" ) ) {
        int irr;
        char marker[32];

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeIrregular ) return ENOTFOR;

        arg[1] = (char*) strtok( NULL, " \t" ); if( arg[1] == 0 ) return EFORMAT;
        arg[2] = (char*) strtok( NULL, " \t" ); if( arg[2] != 0 ) return EMANYARG;

        ks2kimmo( arg[1], wordKimmo );

        r = MAdictIGet( dictID, wordKimmo, &irr );
        if( r == TRUE ) {
            MAdictIBitVector2Marker( irr, marker );
            printf("%x ", irr);
            printf("%s\n", marker );
        }
    }

    else return EUNKNOWNCOM;

    return TRUE;
}


// 
//
//
processAfterTokenization()
{
    char comArg[5][1024];
    int r;

    r = getOneWord( comArg[0], FALSE );
    if( r == 0 ) return 1;

    // IP command : normal
    //
    if( !strcmp( comArg[0], "IP" ) ) {
        int tPos; float freq;

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeNormal ) return ENOTFOR;

        r = getOneWord( comArg[1], TRUE );  if( r == 0 ) return EFORMAT;
        r = getOneWord( comArg[2], FALSE ); if( r == 0 ) return EFORMAT;
        r = getOneWord( comArg[3], FALSE ); if( r == 0 ) return EFORMAT;
        r = getOneWord( comArg[4], FALSE ); if( r != 0 ) return EMANYARG;

        tPos = str2pos( comArg[2] );
        if( tPos < 0 ) return EPOSVAL;
        sscanf( comArg[3], "%f", &freq );

        // error handling in MAdictNInsertPOS() function
        MAdictNInsertPOS( dictID, comArg[1], (enum POS) tPos, freq );
    }

    // DP command : normal
    //
    else if( !strcmp( comArg[0], "DP" ) ) {
        int tPos;

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeNormal ) return ENOTFOR;

        r = getOneWord( comArg[1], TRUE );  if( r == 0 ) return EFORMAT;
        r = getOneWord( comArg[2], FALSE ); if( r == 0 ) return EFORMAT;
        r = getOneWord( comArg[3], FALSE ); if( r != 0 ) return EMANYARG;

        tPos = str2pos( comArg[2] );
        if( tPos < 0 ) return EPOSVAL;
        // error handling in MAdictNDeletePOS() function.
        MAdictNDeletePOS( dictID, comArg[1], (enum POS) tPos );
    }

    // LW command
    //
    else if( !strcmp( comArg[0], "LW" ) ) {
        enum POS pos; float freq; int h, i;

        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeNormal ) return ENOTFOR;

        r = getOneWord( comArg[1], TRUE );  if( r == 0 ) return EFORMAT;
        r = getOneWord( comArg[2], FALSE ); if( r != 0 ) return EMANYARG;

        debugMesg("LW command : dictType is normal.\n");

        for( h = TRUE, i=0; h == TRUE ; i++ ) {
            h = MAdictNGetIthPOS( dictID, comArg[1], i, &pos, &freq );
            if( h == TRUE ) printf("%s %f ", POSstr[pos], freq);
        }
        printf("\n");
    }

    // ZW command
    //
    else if( !strcmp( comArg[0], "ZW" ) ) {
        if( dOpend == FALSE ) return ENOTOPENED;
        if( dt == dictTypeNone ) return ENODICT;
        if( dt != dictTypeNormal ) return ENOTFOR;

        r = getOneWord( comArg[1], TRUE );  if( r == 0 ) return EFORMAT;
        r = getOneWord( comArg[2], FALSE ); if( r != 0 ) return EMANYARG;

        // error handling in MAdictNZapWord() function.
        MAdictNZapWord( dictID, comArg[1] );
    }

    else return EUNKNOWNCOM;

    return TRUE;
}


/* for tokenization handling --------------------------------------------*/

// KSC-5601 -> kimmo code
// if withS is TRUE, then insert SI, SO
getOneWord( s, withS )
    char* s;   // OUT
    int withS; // print SI, SO
{
    register int i;
    int iHeadV = 0;
    int wordToken;           // token number in currently selected word
    static int wordEnd = 0;
    char tLine[256];

    strcpy( s, "" );

    // if cannot make more word, then init static var. and return zero.
    if( wordEnd >= tokenNumber ) {
        wordEnd = 0;
        return 0;
    }

    // make a string with SI(^N), SO(^O) for T_ENG, T_NUM, T_SYM
    // except space character which is used for delimiter.
    wordToken = 0;
    for( i=wordEnd; i<tokenNumber; i++ ) {
        if( !(tokenList[i].tt == tokenS &&
              tokenList[i].string[0] == ' ' && wordToken != 0) )
        {
            switch( tokenList[i].tt ) {
            case tokenS :
            case tokenE :
            case tokenN :
                if( withS == TRUE ) {
                    sprintf(tLine, "%c%s%c",
                      CTRL(N), tokenList[i].string, CTRL(O) );
                    strcat( s, tLine );
                }
                else
                    strcat( s, tokenList[i].string );
                break;
            case tokenK :
                ks2kimmo( tokenList[i].string, tLine );
                strcat( s, tLine );
                break;
            }
            wordToken++;
        }
        else
            break;
    }
    if( wordToken == 0 ) return 0;
    wordEnd = i + 1;  // next token after ' ' token
#if defined(DEBUG)
    { register int j, k;
      char ss[256];
      for( j=0, k=0; j<strlen(s); j++ ) {
        if( s[j] == CTRL(N) ) { ss[k++] = '^'; ss[k++] = 'N'; }
        else if( s[j] == CTRL(O) ) { ss[k++] = '^'; ss[k++] = 'O'; }
        else ss[k++] = s[j];
      }
      ss[k] = '\0';
      debugMesg("getOneWord : %s.\n", ss );
    }
#endif
    return 1;
}



/* misc functions -------------------------------------------------------*/

// printing the usage this program
//
printUsage( fn )
   char* fn;
{
   printf("%s %s (%s)\n", fn, VERSION, REVDATE);
   printf("usage: %s [ -cfdvih ] [ file ... ]\n", fn);
   printf("-c                  read from standard output\n");
   printf("-f file             read from file\n");
   printf("-d dict-name type(n|e|i|u)   dictionary-name and its type\n");
   printf("-v                  dump the whole contents of dictionary\n");
   printf("-i                  ignore error\n");
   printf("-h                  for this screen\n");
   printf("\n");
   printf("possible dictionary-type :\n");
   printf("    n - normal word dictionary\n");
   printf("    e - exceptional word dictionary\n");
   printf("    i - irregular word dictionary\n");
   printf("    u - unknown word guessing dictionary\n\n");

   return FALSE;
}

// processing() error messages
//
printError( eCode )
    int eCode;
{
    switch( eCode ) {
    case ENODICT :
        fprintf(stderr, "No dictionary is specified.\n" ); break;
    case EUNKNOWNCOM :
        fprintf(stderr, "Unknow command.\n" ); break;
    case EFORMAT :
        fprintf(stderr, "Format is incorrect.\n" ); break;
    case EDICTTYPE :
        fprintf(stderr, "Dictionary type is wrong. must be n or e.\n" ); break;
    case EMANYARG :
        fprintf(stderr, "Too many argument for command.\n" ); break;
    case ENOTFOR :
        fprintf(stderr, "Command is not for this type dictionary.\n" ); break;
    case EPOSVAL :
        fprintf(stderr, "Pos value is wrong.\n" ); break;
    case ENOTOPENED :
        fprintf(stderr, "Some commands/options cannot executed when dictionary is not opened.\n" ); break;
    case EIRRMARKER :
        fprintf(stderr, "Irregular marker value is not proper.\n" ); break;
    }
}



/* handling options -----------------------------------------------------*/

char opt2onechar( s )
    char* s;
{
    static char *opts[] = { "-c", "-f", "-d", "-h", "-i", "-v", NULL };
    static char onechar[] = { 'c', 'f', 'd', 'h', 'i', 'v' };

    register int i;

    for( i=0; opts[i] != NULL; i++ )
        if( !strcmp( s, opts[i] ) ) return onechar[i];

}

handleOptions( argc, argv )
    int argc;
    char *argv[];
{
    register int i;
    int isVOption = FALSE;

    // 1st phase processing
    // in case, -v option, it must be processed after processing
    // -d option, so it must be processed in 2nd phase.
    //
    for( i=1; i<argc; i++ ) {

        switch( opt2onechar( argv[i] ) ) {
        case 'c' :
            debugMesg("option : c\n");
            readFrom = RF_STDIN;
            break;

        case 'f' :
            debugMesg("option : f\n");
            if( i+1 >= argc ) return printUsage( argv[0] );
            else {
                readFrom = RF_FILE;
                strcpy( inFn, argv[++i] );
                if( inFn[0] == '-' ) return printUsage( argv[0] );
            }
            break;

        case 'd' :
            debugMesg("option : d\n");
            if( i+2 >= argc ) return printUsage( argv[0] );
            else {
                char comm[128];

                strcpy( dFn, argv[++i] );
                if( dFn[0] == '-' ) return printUsage( argv[0] );

                if( strlen(argv[++i]) != 1 || !strchr( "neui", argv[i][0] ) )
                    return printUsage( argv[0] );
                else {
                    switch( argv[i][0] ) {
                    case 'n' :
                        dt = dictTypeNormal;
                        break;
                    case 'e' :
                        dt = dictTypeExce;
                        break;
                    case 'u' :
                        dt = dictTypeUnknown;
                        break;
                    case 'i' :
                        dt = dictTypeIrregular;
                        break;
                    }
                }

                // error handling in MAdictOpen()
                dictID = MAdictOpen( dFn, O_RDWR | O_CREAT, 0644, dt );
                dOpend = TRUE;
            }
            break;

        case 'v' :
            isVOption = TRUE;
            break;

        case 'i' :
            debugMesg("option : i\n");
            ignoreErr = TRUE;
            break;

        case 'h' :
        default :
            return printUsage( argv[0] );
        }

    }

    // 2nd phase
    //
    if( isVOption == TRUE ) {
        int h;

        h = processOneLine( "V" );
        if( h < 0 ) printError( h );
        return FALSE;  // always termianted the program
    }

    return TRUE;
}
